/*! @file    main.c
 *  @brief   捕获输入
 *  @version 2021-06-30  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_gtim.h"
#include "nccm0670_rcc.h"

/* 捕获通道选择 */
#define CAPTURE_CHANNEL_1 (GTIM_CH_1)

static volatile uint32_t capture_value[2];
static volatile uint8_t capture_flag = 0;
static volatile uint8_t count        = 0;
float capture_period;

void rcc_config(void);
void gpio_config(void);
void timer_base_init(void);
void timer_capture_init(void);
void timer_configuration(void);

int main(void)
{
    rcc_config();
    ncc_eval_com_init();
    gpio_config();
    timer_configuration();
    while (1)
    {
        /* 判断是否捕获到完整的波形周期*/
        if (capture_flag == 1)

        {
            /* 标志位清0 */
            capture_flag = 0;
            /* 波形频率 = 10KHZ * 捕获采样2分频（每两个边沿捕获一次）/定时器捕获值 = 20000HZ / 定时器捕获值 */
            if (capture_value[1] > capture_value[0])
            {
                capture_period = 20000 / (float)(capture_value[1] - capture_value[0]);
            }
            else
            {
                capture_period = 20000 / (float)(capture_value[1] + 10000 - capture_value[0]);
            }
            printf("CapturePeriod: %fHZ\r\n", capture_period);
        }
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_GPIOA);
    RCC_EnablePeripheralClock(RCC_TIM2);
}
void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_VERY_HIGH;
    gpio_cfg.pin           = GPIO_PIN_2;
    gpio_cfg.pull          = GPIO_NOPULL;
    gpio_cfg.alternate     = GPIO_AF1_TIM2;
    GPIO_Init(GPIOA, &gpio_cfg);
}
void timer_base_init(void)
{
    gtim_base_config_t gtim_cfg = {0};
    /* 向上计数模式 */
    gtim_cfg.counter_mode       = GTIM_COUNTER_UP;
    /* 定时器时钟分频 */
    gtim_cfg.clock_division     = GTIM_CLOCK_DIVISION_DIV1;
    /* 计数器预分频系数设为 399 ，4M/(399+1)=10KHZ*/
    gtim_cfg.prescaler          = 399;
    /* 计数器重载值设为 10000 */
    gtim_cfg.period             = 10000;
    GTIM_Init(TIM2, &gtim_cfg);
}
void timer_capture_init(void)
{
    gtim_ic_config_t gtim_ic_cfg = {0};

    /*!< 非反相/上升沿触发 */
    gtim_ic_cfg.polarity         = GTIM_IC_POLARITY_RISING;
    /*!< ICx映射到TIx上*/
    gtim_ic_cfg.selection        = GTIM_IC_SELECTION_DIRECTTI;
    /*!< 预分频系数设为2 ,每2个有效边沿捕获一次*/
    gtim_ic_cfg.prescaler        = GTIM_IC_PSC_DIV2;
    /*!< 滤波器,采样频率=fDTS/1,连续采4个点 */
    gtim_ic_cfg.filter           = GTIM_IC_FILTER_FDIV1_N4;
    GTIM_IC_Init(TIM2, CAPTURE_CHANNEL_1, &gtim_ic_cfg);
}
void timer_configuration(void)
{
    timer_base_init();
    timer_capture_init();

    /*捕获中断使能*/
    GTIM_SetIntMode(TIM2, GTIM_INT_CC1, ENABLE);
    /* 使能定时器 */
    GTIM_Enable(TIM2);
    /*使能定时器中断*/
    NVIC_EnableIRQ(TIM2_IRQn);
}

void TIM2_IRQHandler(void)
{
    /* 通道1捕获中断标志位有效 && 通道1捕获中断使能 */
    if (GTIM_GetIntFlag(TIM2, GTIM_FLAG_CC1))
    {
        if (count == 0)
        {
            /* 第一次捕获 */
            capture_value[0] = GTIM_IC_GetCaptureValue(TIM2, GTIM_CH_1);
            count++;
        }
        else
        {
            /* 第二次捕获 */
            capture_value[1] = GTIM_IC_GetCaptureValue(TIM2, GTIM_CH_1);
            count            = 0;
            /*捕获到完整周期，标志位置位*/
            capture_flag     = 1;
        }
        /*清捕获中断标志*/
        GTIM_ClearStatusFlag(TIM2, GTIM_FLAG_CC1);
    }
}