/*! @file    main.c
 *  @brief   IIC例程
 *  @version 2022-12-13  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_i2c.h"
#include "nccm0670_rcc.h"
#include "i2c_smbus.h"
#include "systick.h"

/* 主机地址配置 */
#define SLAVE_ADDRESS 0x12

void rcc_config(void);
void gpio_config(void);
void i2c_config(void);

#define I2C_DATA_LEN 8
uint8_t i2c_rx_data[I2C_DATA_LEN];

int main(void)
{
    /* 时钟配置 */
    rcc_config();
    /* 引脚配置 */
    gpio_config();
    /* I2C配置 */
    i2c_config();
    /* 调试串口配置 */
    ncc_eval_com_init();
    /* 系统定时器配置 */
    systick_init();

    while (1)
    {
        /* 主机阻塞接收 */
        uint8_t result = i2c_smbus_master_receive(I2C1, SLAVE_ADDRESS, i2c_rx_data, I2C_DATA_LEN, 10000);
        /* 超时处理 */
        if (result != 0U)
        {
            printf("timeout! error %d\r\n", result);
        }
        else
        {
            if (I2C_GetStatusFlag(I2C1, I2C_FLAG_PECERR) != 0U)
            {
                printf("pec! error\r\n");
            }

            /* 打印接收到的数据 */
            printf("receive : ");
            for (uint8_t i = 0; i < I2C_DATA_LEN; i++)
            {
                printf("0x%02x ", i2c_rx_data[i]);
            }
            printf("\r\n");
            delay_ms(1000);
        }
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_GPIOB);
    RCC_EnablePeripheralClock(RCC_I2C1);
}

void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};

    gpio_cfg.pin           = GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_7;
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.output_type   = GPIO_OUTPUT_OPENDRAIN;
    gpio_cfg.pull          = GPIO_PULLUP;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_HIGH;
    gpio_cfg.alternate     = GPIO_AF0_I2C1;
    GPIO_Init(GPIOB, &gpio_cfg);
}

void i2c_config(void)
{
    i2c_config_t i2c_cfg      = {0};

    /* PCLK=4M, I2C 波特率:100K */
    i2c_cfg.timing            = I2C_CONVERT_TIMINGS(0x0, 4, 2, 0xF, 0x13);
    /* 从机地址为7位 */
    i2c_cfg.addressing_mode   = I2C_ADDRESSING_MODE_7BIT;
    /* 禁止双地址模式 */
    i2c_cfg.dual_address_mode = I2C_DUAL_ADDRESS_DISABLE;
    /* 禁止广播模式 */
    i2c_cfg.general_call_mode = I2C_GENERAL_CALL_DISABLE;
    /* 禁止关闭时钟延长 */
    i2c_cfg.no_stretch_mode   = I2C_NO_STRETCH_DISABLE;
    I2C_Init(I2C1, &i2c_cfg);

    /* 使能主机地址 */
    I2C_SetMode(I2C1, I2C_MODE_SMBUS_HOST);
    /* 使能PEC */
    I2C_SMBUS_SetPEC(I2C1, ENABLE);
}
