/*! @file    main.c
 *  @brief   I2C 从机中断方式发送例程
 *  @version 2023-04-23  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_i2c.h"
#include "nccm0670_rcc.h"
#include "systick.h"

/* 从机地址 */
#define SLAVE_ADDRESS  0x82
/* 测试数据长度 */
#define I2C_DATA_LEN   8
/* I2C数据帧最大长度 */
#define MAX_NBYTE_SIZE 255U

void rcc_config(void);
void gpio_config(void);
void i2c_config(void);
uint8_t i2c_slave_transmit_it(I2C_Type *i2cx, uint8_t *data, uint16_t size);

/* 保存返回值 */
uint8_t result;
/* I2C传输数据指针 */
uint8_t *buf_ptr;
/* I2C传输计数器 */
volatile uint16_t xfer_count;
/* I2C传输长度 */
volatile uint16_t xfer_size;
/* I2C传输模式 */
volatile uint16_t xfer_mode;
/* I2C发送数据缓存 */
uint8_t i2c_tx_data[I2C_DATA_LEN];
/* I2C传输结束标志 */
volatile bool xfer_finish;

int main(void)
{
    /* 时钟配置 */
    rcc_config();
    /* 引脚配置 */
    gpio_config();
    /* I2C配置 */
    i2c_config();
    /* 调试串口配置 */
    ncc_eval_com_init();
    /* 系统定时器配置 */
    systick_init();

    for (uint8_t i = 0; i < I2C_DATA_LEN; i++)
    {
        i2c_tx_data[i] = 0x01 + i;
    }
    while (1)
    {
        result = i2c_slave_transmit_it(I2C1, i2c_tx_data, I2C_DATA_LEN);
        if (result != 0U)
        {
            printf("transmit error : %d\r\n", result);
        }
        else
        {
            printf("send successful\r\n");
        }
        /* 等待传输结束 */
        while (xfer_finish == false)
            ;
        xfer_finish = false;
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_GPIOB);
    RCC_EnablePeripheralClock(RCC_I2C1);
}

void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};

    gpio_cfg.pin           = GPIO_PIN_6 | GPIO_PIN_7;
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.output_type   = GPIO_OUTPUT_OPENDRAIN;
    gpio_cfg.pull          = GPIO_PULLUP;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_HIGH;
    gpio_cfg.alternate     = GPIO_AF0_I2C1;
    GPIO_Init(GPIOB, &gpio_cfg);
}

void i2c_config(void)
{
    i2c_config_t i2c_cfg      = {0};

    /* PCLK=4M, I2C 波特率:100K */
    i2c_cfg.timing            = I2C_CONVERT_TIMINGS(0x0, 4, 2, 0xF, 0x13);
    /* 从机地址 */
    i2c_cfg.own_address1      = SLAVE_ADDRESS;
    /* 从机地址为7位 */
    i2c_cfg.addressing_mode   = I2C_ADDRESSING_MODE_7BIT;
    /* 禁止双地址模式 */
    i2c_cfg.dual_address_mode = I2C_DUAL_ADDRESS_DISABLE;
    /* 禁止广播模式 */
    i2c_cfg.general_call_mode = I2C_GENERAL_CALL_DISABLE;
    /* 禁止关闭时钟延长 */
    i2c_cfg.no_stretch_mode   = I2C_NO_STRETCH_DISABLE;
    I2C_Init(I2C1, &i2c_cfg);
    /* 打开I2C中断 */
    NVIC_EnableIRQ(I2C1_IRQn);
}

/**
 * @brief 从机中断方式发送数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] data             数据缓存
 * @param[in] size             数据长度
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_slave_transmit_it(I2C_Type *i2cx, uint8_t *data, uint16_t size)
{
    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 总线忙 */
    if (I2C_GetStatusFlag(i2cx, I2C_FLAG_BUSY))
    {
        return 2;
    }
    /* 开启ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_ACK);

    buf_ptr    = data;
    xfer_count = size;
    xfer_size  = xfer_count;
    I2C_SetIntMode(i2cx, I2C_INT_TXIE, ENABLE);
    I2C_SetIntMode(i2cx, I2C_INT_ADDRIE, ENABLE);
    I2C_SetIntMode(i2cx, I2C_INT_STOPIE, ENABLE);

    return 0;
}

void I2C1_IRQHandler(void)
{
    if (I2C_GetIntFlag(I2C1, I2C_FLAG_ADDR))
    {
        I2C_ClearStatusFlag(I2C1, I2C_FLAG_ADDR);
    }
    else if (I2C_GetIntFlag(I2C1, I2C_FLAG_TXIS))
    {
        I2C_TransmitData8(I2C1, *buf_ptr);

        buf_ptr++;
        xfer_size--;
        if (xfer_size == 0)
        {
            I2C_SetIntMode(I2C1, I2C_INT_TXIE, DISABLE);
        }
    }
    else
    {
    }

    if (I2C_GetIntFlag(I2C1, I2C_FLAG_STOP))
    {
        I2C_ClearStatusFlag(I2C1, I2C_FLAG_STOP);
        xfer_finish = true;
    }
}