/*! @file    main.c
 *  @brief   SPI例程
 *  @version 2021-07-19  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_rcc.h"
#include "nccm0670_spi.h"
#include "systick.h"

#define BUFFER_LEN 10U
uint8_t s_send_buffer[BUFFER_LEN] = "123456789";
uint8_t s_recv_buffer[BUFFER_LEN];
uint8_t s_send_counter, s_recv_counter;
uint8_t s_complete_flag;

void rcc_config(void);
void gpio_config(void);
void spi_config(void);

int main(void)
{
    /* 时钟配置 */
    rcc_config();
    /* 引脚配置 */
    gpio_config();
    /* SPI配置 */
    spi_config();
    /* 调试串口配置 */
    ncc_eval_com_init();
    /* 系统定时器配置 */
    systick_init();

    printf("SPI master fullduplex communication\r\n");

    while (1)
    {
        if (s_complete_flag)
        {
            s_complete_flag = 0;
            printf("receive : ");
            for (uint8_t i = 0; i < BUFFER_LEN; i++)
            {
                printf("0x%02x ", s_recv_buffer[i]);
            }
            printf("\r\n");
            delay_ms(1000);
            /* 重新开始下一次通信 */
            SPI_SetIntMode(SPI1, SPI_INT_TXE, ENABLE);
        }
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_GPIOB);
    RCC_EnablePeripheralClock(RCC_SPI1);
}

void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};

    gpio_cfg.pin           = GPIO_PIN_2 | GPIO_PIN_3 | GPIO_PIN_4 | GPIO_PIN_5;
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_HIGH;
    gpio_cfg.output_type   = GPIO_OUTPUT_PUSHPULL;
    gpio_cfg.pull          = GPIO_PULLDOWN;
    gpio_cfg.alternate     = GPIO_AF1_SPI1;
    GPIO_Init(GPIOB, &gpio_cfg);
}

void spi_config(void)
{
    spi_config_t spi_cfg = {0};

    /* 主机模式 */
    spi_cfg.mode         = SPI_MODE_MASTER;
    /* Motorola模式 */
    spi_cfg.protocol     = SPI_PROTOCOL_MOTOROLA;
    /* 双线全双工 */
    spi_cfg.direction    = SPI_DIRECTION_2LINES;
    /* 8位帧长 */
    spi_cfg.frame_size   = SPI_FRAME_SIZE_8BITS;
    /* SPI时钟极性 */
    spi_cfg.clk_polarity = SPI_SCK_ACTIVE_LOW;
    /* SPI时钟相位 */
    spi_cfg.clk_phase    = SPI_SCK_PHASE_2ND_EDGE;
    /* NSS选择 */
    spi_cfg.nss          = SPI_NSS_HARD;
    /* NSS极性 */
    spi_cfg.nss_polarity = SPI_NSS_ACTIVE_LOW;
    /* SPI时钟分频 */
    spi_cfg.prescaler    = SPI_BAUDRATE_PRESCALER(2);
    /* 字节序配置 */
    spi_cfg.byte_order   = SPI_BYTE_ORDER_MOTOROLA;
    /* 比特序配置 */
    spi_cfg.bit_order    = SPI_LSB_FIRST;
    /* 滤波器使能 */
    spi_cfg.filter       = SPI_FILTER_DISABLE;
    SPI_Init(SPI1, &spi_cfg);

    SPI_SetIntMode(SPI1, SPI_INT_TXE, ENABLE);
    SPI_SetIntMode(SPI1, SPI_INT_RXNE, ENABLE);

    SPI_Enable(SPI1);

    NVIC_SetPriority(SPI1_IRQn, 5);
    NVIC_EnableIRQ(SPI1_IRQn);
}

void SPI1_IRQHandler(void)
{
    if (SPI_GetIntFlag(SPI1, SPI_FLAG_TXE))
    {
        SPI_TransmitData(SPI1, s_send_buffer[s_send_counter++]);
        if (s_send_counter == BUFFER_LEN)
        {
            s_send_counter = 0;
            SPI_SetIntMode(SPI1, SPI_INT_TXE, DISABLE);
        }
    }
    if (SPI_GetIntFlag(SPI1, SPI_FLAG_RXNE))
    {
        s_recv_buffer[s_recv_counter++] = SPI_ReceiveData(SPI1);
        if (s_recv_counter == BUFFER_LEN)
        {
            s_recv_counter  = 0;
            s_complete_flag = 1;
        }
    }
}
