/*! @file    main.c
 *  @brief   USART Irda发送例程
 *  @version 2023-05-10  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_gpio.h"
#include "nccm0670_gtim.h"
#include "nccm0670_rcc.h"
#include "nccm0670_syscfg.h"
#include "nccm0670_usart.h"
#include "systick.h"

void rcc_config(void);
void gpio_config(void);
void gtim_config(void);
void usart_config(void);

int main(void)
{
    /* 初始化RCC */
    rcc_config();
    /* 初始化GPIO */
    gpio_config();
    /* 初始化GTIM */
    gtim_config();
    /* 初始化USART */
    usart_config();
    /* 初始化系统定时器 */
    systick_init();
    while (1)
    {
        USART_TransmitData(USART1, 0x55);
        delay_ms(20);
    }
}

void usart_config(void)
{
    usart_base_cfg_t usart_cfg      = {0};

    /* 通信波特率 */
    usart_cfg.baudrate              = 1200;
    /* 帧长度 */
    usart_cfg.datawidth             = USART_DATAWIDTH_8B;
    /* 1位停止位(0.5和1相同、1.5和2相同) */
    usart_cfg.stopbits              = USART_STOPBITS_1;
    /* 奇偶校验位 */
    usart_cfg.parity                = USART_PARITY_NONE;
    /* 传输方向 */
    usart_cfg.transfer_direction    = USART_DIRECTION_TX_RX;
    USART_Init(USART1, &usart_cfg);
    USART_Enable(USART1);

    USART_SetIrdaPowerMode(USART1, USART_IRDA_POWER_NORMAL);
    /* Irda功能使能 */
    USART_SetIrda(USART1, ENABLE);
    /* IR_OUT输出极性不反转 */
    SYSCFG_SetIrPolarity(SYSCFG_IR_POL_NONINVERTED);
    /* IR_OUT输出使能 */
    SYSCFG_SetIrMode(ENABLE);
}

void rcc_config(void)
{
    RCC_SetHsiDiv(RCC_HSI_DIV6);

    RCC_EnablePeripheralClock(RCC_SYSCFG);
    RCC_EnablePeripheralClock(RCC_TIM10);
    RCC_EnablePeripheralClock(RCC_USART1);
    RCC_EnablePeripheralClock(RCC_GPIOB);
}

void gpio_config(void)
{
    gpio_config_t gpio_cfg = {0};

    gpio_cfg.pin           = GPIO_PIN_15;
    gpio_cfg.mode          = GPIO_MODE_ALTERNATE;
    gpio_cfg.output_type   = GPIO_OUTPUT_OPENDRAIN;
    gpio_cfg.pull          = GPIO_PULLUP;
    gpio_cfg.speed         = GPIO_SPEED_FREQ_LOW;
    gpio_cfg.alternate     = GPIO_AF1_IR;
    GPIO_Init(GPIOB, &gpio_cfg);
}

void gtim_config(void)
{
    /* 载波为38Khz，系统时钟为8M， TIM10 周期为 8000/38=210 */
    gtim_base_config_t cfg = {
        .counter_mode   = GTIM_COUNTER_UP,
        .period         = 210,
        .prescaler      = 0,
        .clock_division = GTIM_CLOCK_DIVISION_DIV1,
    };
    GTIM_Init(TIM10, &cfg);

    gtim_oc_config_t occfg = {
        .state    = GTIM_OC_OUTPUT_ENABLE,
        .polarity = GTIM_OC_POLARITY_HIGH,
    };
    GTIM_OC_Init(TIM10, GTIM_CH_1, &occfg);
    GTIM_OC_SetMode(TIM10, GTIM_CH_1, GTIM_OC_MODE_PWM1);
    GTIM_OC_SetCompareValue(TIM10, GTIM_CH_1, 52);
    GTIM_OC_SetPreload(TIM10, GTIM_CH_1, ENABLE);
    GTIM_SetArrPreload(TIM10, ENABLE);
    GTIM_Enable(TIM10);
}