/*! @file    nccm0670_exti.h
 *  @brief   EXTI固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_EXTI_H
#define NCC0670_EXTI_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_exti
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define EXTI_PROPERTY_SHIFT 24U
#define EXTI_DIRECT         (0x01UL << EXTI_PROPERTY_SHIFT)
#define EXTI_CONFIG         (0x02UL << EXTI_PROPERTY_SHIFT)
#define EXTI_GPIO           ((0x04UL << EXTI_PROPERTY_SHIFT) | EXTI_CONFIG)
#define EXTI_RESERVED       (0x08UL << EXTI_PROPERTY_SHIFT)
#define EXTI_PROPERTY_MASK  (EXTI_DIRECT | EXTI_CONFIG | EXTI_GPIO)

#define EXTI_PIN_MASK 0x0000001FU

/**
 * @brief EXTI 线号
 */
typedef enum
{
    EXTI_LINE_0  = EXTI_GPIO | 0x00U,     /*!< EXTI线 0 */
    EXTI_LINE_1  = EXTI_GPIO | 0x01U,     /*!< EXTI线 1 */
    EXTI_LINE_2  = EXTI_GPIO | 0x02U,     /*!< EXTI线 2 */
    EXTI_LINE_3  = EXTI_GPIO | 0x03U,     /*!< EXTI线 3 */
    EXTI_LINE_4  = EXTI_GPIO | 0x04U,     /*!< EXTI线 4 */
    EXTI_LINE_5  = EXTI_GPIO | 0x05U,     /*!< EXTI线 5 */
    EXTI_LINE_6  = EXTI_GPIO | 0x06U,     /*!< EXTI线 6 */
    EXTI_LINE_7  = EXTI_GPIO | 0x07U,     /*!< EXTI线 7 */
    EXTI_LINE_8  = EXTI_GPIO | 0x08U,     /*!< EXTI线 8 */
    EXTI_LINE_9  = EXTI_GPIO | 0x09U,     /*!< EXTI线 9 */
    EXTI_LINE_10 = EXTI_GPIO | 0x0AU,     /*!< EXTI线 10 */
    EXTI_LINE_11 = EXTI_GPIO | 0x0BU,     /*!< EXTI线 11 */
    EXTI_LINE_12 = EXTI_GPIO | 0x0CU,     /*!< EXTI线 12 */
    EXTI_LINE_13 = EXTI_GPIO | 0x0DU,     /*!< EXTI线 13 */
    EXTI_LINE_14 = EXTI_GPIO | 0x0EU,     /*!< EXTI线 14 */
    EXTI_LINE_15 = EXTI_GPIO | 0x0FU,     /*!< EXTI线 15 */
    EXTI_LINE_16 = EXTI_CONFIG | 0x10U,   /*!< EXTI线 16 */
    EXTI_LINE_17 = EXTI_DIRECT | 0x11U,   /*!< EXTI线 17 */
    EXTI_LINE_18 = EXTI_DIRECT | 0x12U,   /*!< EXTI线 18 */
    EXTI_LINE_19 = EXTI_DIRECT | 0x13U,   /*!< EXTI线 19 */
    EXTI_LINE_20 = EXTI_DIRECT | 0x14U,   /*!< EXTI线 20 */
    EXTI_LINE_21 = EXTI_RESERVED | 0x15U, /*!< EXTI线 21 */
    EXTI_LINE_22 = EXTI_RESERVED | 0x16U, /*!< EXTI线 22 */
    EXTI_LINE_23 = EXTI_RESERVED | 0x17U  /*!< EXTI线 23 */
} exti_line_t;

/**
 * @brief EXTI模式
 */
typedef enum
{
    EXTI_MODE_NONE,
    EXTI_MODE_INTERRUPT, /*!< 中断模式 */
    EXTI_MODE_EVENT      /*!< 事件模式 */
} exti_mode_t;

/**
 * @brief EXTI触发沿
 */
typedef enum
{
    EXTI_TRIGGER_NONE,          /*!< 无触发 */
    EXTI_TRIGGER_RISING,        /*!< 上升沿触发 */
    EXTI_TRIGGER_FALLING,       /*!< 下降沿触发 */
    EXTI_TRIGGER_RISING_FALLING /*!< 上升沿和下降沿触发 */
} exti_trigger_t;

typedef enum
{
    EXTI_GPIOA, /*!< EXTI 端口 A */
    EXTI_GPIOB, /*!< EXTI 端口 B */
    EXTI_GPIOC, /*!< EXTI 端口 C */
#ifdef GPIOD
    EXTI_GPIOD, /*!< EXTI 端口 D */
#endif
#ifdef GPIOF
    EXTI_GPIOF = 0x05U, /*!< EXTI 端口 F */
#endif
} exti_gpio_sel_t;

/**
 * @brief EXTI配置结构体
 */
typedef struct
{
    exti_mode_t mode;       /*!< 中断和事件模式 */
    exti_trigger_t trigger; /*!< 触发边沿 */
    exti_gpio_sel_t gpio_sel;
} exti_config_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

void EXTI_SetConfigLine(exti_line_t line, exti_config_t *config);
void EXTI_ClearConfigLine(exti_line_t line);
bool EXTI_GetPending(exti_line_t line, exti_trigger_t edge);
void EXTI_ClearPending(exti_line_t line, exti_trigger_t edge);
void EXTI_GenerateSwi(exti_line_t line);

#if defined(__cplusplus)
}
#endif

/*! @} */
#endif /* NCC0670_EXTI_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
