/*! @file    nccm0670_lcd.h
 *  @brief   LCD固件库
 *  @version 2021-04-02  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_LCD_H
#define NCC0670_LCD_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_lcd
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief   LCD驱动模式选择
 */
typedef enum
{
    LCD_DRIVE_MODE_CAPACITIVE_MODE0 = LCD_CR_TSEL(1),                        /*!< 电容偏压型 */
    LCD_DRIVE_MODE_CAPACITIVE_MODE1 = LCD_CR_PPSEL(1),                       /*!< 内建电压稳定器型  */
    LCD_DRIVE_MODE_RESISTIVE_NETWORK_450K = 0U,                              /*!< 偏置电阻总和为450k*/
    LCD_DRIVE_MODE_RESISTIVE_NETWORK_990K = LCD_CR_RLCD(1),                  /*!< 偏置电阻总和为990k*/
    LCD_DRIVE_MODE_RESISTIVE_NETWORK_60K = LCD_CR_MOD(1),                    /*!< 偏置电阻总和为60k */
    LCD_DRIVE_MODE_RESISTIVE_NETWORK_AUTO0 = LCD_CR_MOD(2),                  /*!< 偏置电阻总和自动在60k和450k间切换*/
    LCD_DRIVE_MODE_RESISTIVE_NETWORK_AUTO1 = LCD_CR_MOD(2) | LCD_CR_RLCD(1), /*!< 偏置电阻总和自动在60k和990k间切换*/
    LCD_DRIVE_MODE_LED_MODE0 = LCD_CR_TSEL(2),                               /*!< LED驱动，COM低电平有效 */
    LCD_DRIVE_MODE_LED_MODE1 = LCD_CR_TSEL(2) | LCD_CR_LEDPOL(1)             /*!< LED驱动，COM高电平有效 */
} lcd_drive_mode_t;

/*!
 * @brief   对比度选择，用于电阻分压驱动LCD
 */
typedef enum
{
    LCD_CONTRAST_LEVEL_0,  /*!< 最大电压 > 5.17V */
    LCD_CONTRAST_LEVEL_1,  /*!< 最大电压 = 5.1V  */
    LCD_CONTRAST_LEVEL_2,  /*!< 最大电压 = 4.95V */
    LCD_CONTRAST_LEVEL_3,  /*!< 最大电压 = 4.80V */
    LCD_CONTRAST_LEVEL_4,  /*!< 最大电压 = 4.65V */
    LCD_CONTRAST_LEVEL_5,  /*!< 最大电压 = 4.50V */
    LCD_CONTRAST_LEVEL_6,  /*!< 最大电压 = 4.35V */
    LCD_CONTRAST_LEVEL_7,  /*!< 最大电压 = 4.20V */
    LCD_CONTRAST_LEVEL_8,  /*!< 最大电压 = 4.05V */
    LCD_CONTRAST_LEVEL_9,  /*!< 最大电压 = 3.90V */
    LCD_CONTRAST_LEVEL_10, /*!< 最大电压 = 3.75V */
    LCD_CONTRAST_LEVEL_11, /*!< 最大电压 = 3.60V */
    LCD_CONTRAST_LEVEL_12, /*!< 最大电压 = 3.45V */
    LCD_CONTRAST_LEVEL_13, /*!< 最大电压 = 3.30V */
    LCD_CONTRAST_LEVEL_14, /*!< 最大电压 = 3.15V */
    LCD_CONTRAST_LEVEL_15  /*!< 最大电压 < 3.07V */
} lcd_contrast_t;

/*!
 * @brief   充电时间控制
 */
typedef enum
{
    LCD_CHARGE_TIME_0, /*!< 1/4 com周期  */
    LCD_CHARGE_TIME_1, /*!< 1/8 com周期  */
    LCD_CHARGE_TIME_2, /*!< 1/16 com周期  */
    LCD_CHARGE_TIME_3  /*!< 1/32 com周期  */
} lcd_charge_time_t;

/*!
 * @brief   占空比选择
 */
typedef enum
{
    LCD_DUTY_1_4, /*!< 1/4占空比，1/3偏置 */
    LCD_DUTY_1_8  /*!< 1/8占空比，只能用于LED驱动模式；或LCD驱动时：1/5占空比，1/3偏置 */
} lcd_duty_t;

/*!
 * @brief   闪烁模式
 */
typedef enum
{
    LCD_BLINK_MODE_DISABLE,      /*!< 禁止闪烁 */
    LCD_BLINK_MODE_1SEG_1COM,    /*!< 指定SEG和COM闪烁 */
    LCD_BLINK_MODE_1SEG_ALLCOM,  /*!< 指定SEG(所有COM)闪烁 */
    LCD_BLINK_MODE_ALLSEG_ALLCOM /*!< 所有段都闪烁 */
} lcd_blink_mode_t;

/*!
 * @brief   闪烁频率
 */
typedef enum
{
    LCD_BLINK_FREQ_0P5HZ, /*!< 0.5Hz */
    LCD_BLINK_FREQ_1HZ,   /*!< 1Hz */
    LCD_BLINK_FREQ_2HZ,   /*!< 2Hz */
    LCD_BLINK_FREQ_4HZ    /*!< 4Hz */
} lcd_blink_freq_t;

/*!
 * @brief   LCD配置结构体
 */
typedef struct
{
    lcd_drive_mode_t drive_mode;   /*!< LCD 工作模式 */
    lcd_contrast_t contrast;       /*!< 对比度选择 */
    lcd_charge_time_t charge_time; /*!< 充电时间 */
    lcd_duty_t duty;               /*!< 占空比 */
    lcd_blink_mode_t blink_mode;   /*!< 闪烁模式 */
    lcd_blink_freq_t blink_freq;   /*!< 闪烁频率 */
} lcd_config_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

void LCD_Init(const lcd_config_t *config);
void LCD_DeInit(void);
void LCD_Enable();
void LCD_Disable();
void LCD_ConfigBlink(lcd_blink_mode_t mode, lcd_blink_freq_t freq);
void LCD_ConfigContrast(lcd_contrast_t contrast);
void LCD_Write(uint8_t regidx, uint8_t data);
void LCD_Clear(void);
void LCD_SetComAndSegMask(uint32_t com_mask, uint32_t seg_mask_low, uint32_t seg_mask_high);
void LCD_SetBlinkPixel(uint8_t com, uint8_t seg);

#if defined(__cplusplus)
}
#endif
/*! @} */

#endif /* NCC0670_LCD_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
