/*! @file    nccm0670_lpuart.h
 *  @brief   LPUART固件库
 *  @version 2021-07-21  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_LPUART_H
#define NCC0670_LPUART_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_lpuart
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/**
 * @brief 奇偶校验
 */
typedef enum
{
    LPUART_PARITY_NONE,                                         /*!< 关闭奇偶校验 */
    LPUART_PARITY_EVEN = LPUART_CR1_PCE_Msk,                    /*!< 偶校验 */
    LPUART_PARITY_ODD  = LPUART_CR1_PCE_Msk | LPUART_CR1_PS_Msk /*!< 奇校验 */
} lpuart_parity_t;

/**
 * @brief 数据长度
 * @note 数据位长度包含奇偶校验位
 */
typedef enum
{
    LPUART_DATAWIDTH_7B = LPUART_CR1_M1_Msk, /*!< 1 个起始位， 7 个数据位， n 个停止位 */
    LPUART_DATAWIDTH_8B = 0x00000000UL,      /*!< 1 个起始位， 8 个数据位， n 个停止位 */
    LPUART_DATAWIDTH_9B = LPUART_CR1_M0_Msk  /*!< 1 个起始位， 9 个数据位， n 个停止位 */
} lpuart_datawidth_t;

/**
 * @brief 停止位
 */
typedef enum
{
    LPUART_STOPBITS_1, /*!< 1 个停止位 */
    LPUART_STOPBITS_2  /*!< 2 个停止位 */
} lpuart_stopbits_t;

/**
 * @brief 传输方向
 */
typedef enum
{
    LPUART_DIRECTION_NONE  = 0x00000000UL,                         /*!< 关闭发送和接收 */
    LPUART_DIRECTION_RX    = LPUART_CR1_RE_Msk,                    /*!< 只接收 */
    LPUART_DIRECTION_TX    = LPUART_CR1_TE_Msk,                    /*!< 只发送 */
    LPUART_DIRECTION_TX_RX = LPUART_CR1_RE_Msk | LPUART_CR1_TE_Msk /*!< 发送和接收 */
} lpuart_direction_t;

/**
 * @brief RX/TX引脚交换
 */
typedef enum
{
    LPUART_TXRX_STANDARD, /*!< TX/RX 引脚采用默认定义 */
    LPUART_TXRX_SWAPPED   /*!< TX 和 RX 引脚交换 */
} lpuart_pin_swap_t;

/**
 * @brief RX/TX引脚反向
 */
typedef enum
{
    LPUART_PIN_LEVEL_STANDARD, /*!< 引脚信号使用标准逻辑电平 */
    LPUART_PIN_LEVEL_INVERTED  /*!< 引脚信号值取反          */
} lpuart_pin_level_t;

/**
 * @brief RX/TX引脚SWAP
 */
typedef enum
{
    LPUART_RXTX_STANDARD, /*!< 引脚使用标准方式 */
    LPUART_RXTX_SWAP      /*!< 引脚交换RX TX    */
} lpuart_rxtx_swap_t;

/**
 * @brief 传输比特顺序
 */
typedef enum
{
    LPUART_BITORDER_LSBFIRST, /*!< 在起始位之后，首先使用数据位 0 发送/接收数据 */
    LPUART_BITORDER_MSBFIRST  /*!< 在起始位之后，首先使用 MSB（位 7/8）发送/接收数据 */
} lpuart_bitorder_t;

/**
 * @brief 7 位地址检测/4 位地址检测
 */
typedef enum
{
    LPUART_ADDRESS_DETECT_4B, /*!< 4 位地址检测 */
    LPUART_ADDRESS_DETECT_7B  /*!< 7 位地址检测（在 8 位数据模式下） */
} lpuart_address_detect_t;

/**
 * @brief LPUART 静默模式的唤醒方法
 */
typedef enum
{
    LPUART_WAKEUP_IDLELINE,   /*!<  空闲线路 */
    LPUART_WAKEUP_ADDRESSMARK /*!<  地址标记 */
} lpuart_wakeup_method_t;

/**
 * @brief LPUART stop mode
 */
typedef enum
{
    LPUART_STOP_MODE_DISABLE, /*!< disable stop mode */
    LPUART_STOP_MODE_ENABLE   /*!<enable stop mode  */
} lpuart_stop_mode_t;

/**
 * @brief 中断 使能/标志/清除位
 */
typedef enum
{

    LPUART_INT_IDLE = LPUART_CR1_IDLEIE_Msk,             /*!< IDLE 帧中断 */
    LPUART_INT_CM   = LPUART_CR1_CMIE_Msk,               /*!< 字符匹配中断*/
    LPUART_INT_TXE  = LPUART_CR1_TXEIE_Msk,              /*!< TXEIF 中断 */
    LPUART_INT_TC   = LPUART_CR1_TCIE_Msk,               /*!< 传输完成中断 */
    LPUART_INT_RXNE = LPUART_CR1_RXNEIE_Msk,             /*!< RXNEIF 标志中断 */
    LPUART_INT_PE   = LPUART_CR1_PEIE_Msk,               /*!< PE奇偶校验错误中断 */
    LPUART_INT_ERR  = 0x80000000U | LPUART_CR3_EIE_Msk,  /*!< 错误中断 */
    LPUART_INT_WUF  = 0x80000000U | LPUART_CR3_WUFIE_Msk /*!< 从低功耗模式唤醒中断 */
} lpuart_interrupt_t;

/**
 * @brief 中断和状态 标志位
 */
typedef enum
{
    LPUART_FLAG_IDLE = LPUART_SR_IDLE_Msk, /*!< 中断：检测到空闲线路 */
    LPUART_FLAG_WUF  = LPUART_SR_WUF_Msk,  /*!< 中断：接收器检测到静默模式唤醒事件标志位 */
    LPUART_FLAG_RWU  = LPUART_SR_RWU_Msk,  /*!< 接收器静默模式状态位 */
    LPUART_FLAG_CMF  = LPUART_SR_CMF_Msk,  /*!< 中断：字符匹配标志 */
    LPUART_FLAG_SBKF = LPUART_SR_SBKF_Msk, /*!< 中断：Send break flag */
    LPUART_FLAG_TC   = LPUART_SR_TC_Msk,   /*!< 中断：发送完成 */
    LPUART_FLAG_TXE  = LPUART_SR_TXE_Msk,  /*!< 中断：发送数据寄存器为空 */
    LPUART_FLAG_RXNE = LPUART_SR_RXNE_Msk, /*!< 中断：读取数据寄存器不为空 */
    LPUART_FLAG_ORE  = LPUART_SR_ORE_Msk,  /*!< 中断：溢出错误 */
    LPUART_FLAG_FE   = LPUART_SR_FE_Msk,   /*!< 中断：帧错误 */
    LPUART_FLAG_PE   = LPUART_SR_PE_Msk,   /*!< 中断：奇偶校验错误 */
    LPUART_FLAG_BUSY = LPUART_SR_BUSY_Msk, /*!< 忙标志 */
    LPUART_FLAG_ABRF = LPUART_SR_ABRF_Msk, /*!< 自动波特率标志 */
    LPUART_FLAG_ABRE = LPUART_SR_ABRE_Msk  /*!< 自动波特率错误标志 */
} lpuart_status_flag_t;

/**
 * @brief LPUART 初始化配置
 */
typedef struct
{
    uint32_t baudrate;                     /*!< 波特率 */
    lpuart_datawidth_t datawidth;          /*!< 帧长度 */
    lpuart_stopbits_t stopbits;            /*!< 停止位 */
    lpuart_parity_t parity;                /*!< 奇偶校验位 */
    lpuart_direction_t transfer_direction; /*!< 传输方向 */
    lpuart_rxtx_swap_t rxtx_swap;          /*!< RX/TX SWAP */
    lpuart_stop_mode_t stop_mode;          /*!< stop mode */
} lpuart_config_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif
/* LPUART 通用 API */
void LPUART_DeInit(LPUART_Type *const base);
void LPUART_Init(LPUART_Type *const base, const lpuart_config_t *config);
void LPUART_SetBaudRate(LPUART_Type *const base, uint32_t baud);
void LPUART_Enable(LPUART_Type *const base);
void LPUART_Disable(LPUART_Type *const base);
void LPUART_SetTransferDirection(LPUART_Type *const base, lpuart_direction_t direction);
void LPUART_SetStopMode(LPUART_Type *const base, bool enable);
void LPUART_SetTransferBitOrder(LPUART_Type *const base, lpuart_bitorder_t bitorder);
void LPUART_SetTxRxSwap(LPUART_Type *const base, lpuart_pin_swap_t swap);
void LPUART_SetRxPinLevel(LPUART_Type *const base, lpuart_pin_level_t inv);
void LPUART_SetTxPinLevel(LPUART_Type *const base, lpuart_pin_level_t inv);
void LPUART_SetRxTimeoutMode(LPUART_Type *const base, bool enable);
void LPUART_SetRxTimeout(LPUART_Type *const base, uint32_t timeout);
void LPUART_TransmitData(LPUART_Type *const base, uint32_t value);
uint16_t LPUART_ReceiveData(LPUART_Type *const base);
void LPUART_RequestRxDataFlush(LPUART_Type *const base);
void LPUART_RequestBreakSending(LPUART_Type *const base);

/* 自动波特率侦测 API */
void LPUART_SetAutoBaudRate(LPUART_Type *const base, bool enable);
void LPUART_RequestAutoBaudRate(LPUART_Type *const base);

/* 字符匹配　API */
void LPUART_SetMatchMode(LPUART_Type *const base, bool enable);
void LPUART_SetMatchCharacter(LPUART_Type *const base, uint8_t ch);

/* 多机通信 API */
void LPUART_ConfigNodeAddress(LPUART_Type *const base, lpuart_address_detect_t address_len, uint8_t node_address);
void LPUART_SetMute(LPUART_Type *const base, bool enable);
void LPUART_RequestEnterMuteMode(LPUART_Type *const base);
void LPUART_SetWakeUpMethod(LPUART_Type *const base, lpuart_wakeup_method_t method);

/* 半双工通信 API */
void LPUART_SetHalfDuplex(LPUART_Type *const base, bool enable);

/* 状态与中断 API */
bool LPUART_GetStatusFlag(LPUART_Type *const base, lpuart_status_flag_t flag);
void LPUART_SetIntMode(LPUART_Type *const base, lpuart_interrupt_t interrupt, bool enable);
bool LPUART_GetIntFlag(LPUART_Type *const base, lpuart_status_flag_t interrupt);
void LPUART_ClearStatusFlag(LPUART_Type *const base, lpuart_status_flag_t interrupt);

#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_LPUART_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
