/*! @file    nccm0670_rcc.h
 *  @brief   RCC固件库
 *  @version 2021-04-06  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_RCC_H
#define NCC0670_RCC_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_rcc
 * @{
 */
/*******************************************************************************
 * Definitions
 ******************************************************************************/
/* 寄存器索引 ENR + RSTR + SMENR 偏移地址*/
#define RCC_REGIDX_AHB   (0x1C1424U)
#define RCC_REGIDX_APB1  (0x201828U)
#define RCC_REGIDX_APB2  (0x4C4850U)
/* 寄存器索引 偏移地址*/
#define RCC_REGIDX_CR    (0x00U)
#define RCC_REGIDX_LSICR (0x34U)
#define RCC_REGIDX_LSECR  (0x38U)

/*!
 * @name 复位原因标志
 * @{
 */
#define RCC_FLAG_OBRST   (RCC_RSTID_CFG_RSTF_Msk) /*!< OPTION BYTE 重新加载复位标志 */
#define RCC_FLAG_PINRST  (RCC_RSTID_PINRSTF_Msk)  /*!< 引脚复位标志 */
#define RCC_FLAG_PWRRST  (RCC_RSTID_PWRRSTF_Msk)  /*!< BOR 或 POR/PDR 标志 */
#define RCC_FLAG_SFTRST  (RCC_RSTID_SFTRSTF_Msk)  /*!< 软件复位标志 */
#define RCC_FLAG_IWDGRST (RCC_RSTID_IWDGRSTF_Msk) /*!< 独立看门狗复位标志 */
#define RCC_FLAG_WWDGRST (RCC_RSTID_WWDGRSTF_Msk) /*!< 窗口看门狗复位标志 */
/*! @} */
/*!
 * @brief  时钟类型
 */
typedef enum
{
    RCC_CLOCK_SYSCLK, /*!< 系统时钟 */
    RCC_CLOCK_HCLK,   /*!< AHB时钟 */
    RCC_CLOCK_PCLK1   /*!< APB1时钟 */
} rcc_clock_type_t;

/*!
 * @brief
 */
typedef enum
{
    RCC_OSC_HSI = REG_IDX_BIT(RCC_REGIDX_CR, RCC_CR_HSION_Pos),
    RCC_OSC_LSE = REG_IDX_BIT(RCC_REGIDX_LSECR, RCC_LSECR_LSEON_Pos),
    RCC_OSC_LSI = REG_IDX_BIT(RCC_REGIDX_LSICR, RCC_LSICR_LSION_Pos),
} rcc_osc_type_t;

/*!
 * @brief HSI 时钟分频系数
 */
typedef enum
{
    RCC_HSI_DIV1,  /*!< HSI 不分频 */
    RCC_HSI_DIV2,  /*!< HSI 2分频*/
    RCC_HSI_DIV4,  /*!< HSI 4分频*/
    RCC_HSI_DIV6,  /*!< HSI 6分频*/
    RCC_HSI_DIV12, /*!< HSI 12分频 */
    RCC_HSI_DIV24, /*!< HSI 24分频 */
    RCC_HSI_DIV48, /*!< HSI 48分频 */
    RCC_HSI_DIV96  /*!< HSI 96分频 */
} rcc_hsi_div_t;

/*!
 * @brief HCLK分频系数
 */
typedef enum
{
    RCC_HCLK_DIV1,     /*!< HCLK 不分频 */
    RCC_HCLK_DIV2 = 4, /*!< HCLK 2分频 */
    RCC_HCLK_DIV4,     /*!< HCLK 4分频 */
    RCC_HCLK_DIV8,     /*!< HCLK 8分频 */
    RCC_HCLK_DIV16     /*!< HCLK 16分频 */
} rcc_hclk_div_t;

/*!
 * @brief  系统时钟分频系数
 */
typedef enum
{
    RCC_SYSCLK_DIV1,      /*!< SYSCLK 不分频 */
    RCC_SYSCLK_DIV2 = 8u, /*!< SYSCLK 2分频 */
    RCC_SYSCLK_DIV4,      /*!< SYSCLK 4分频 */
    RCC_SYSCLK_DIV8,      /*!< SYSCLK 8分频 */
    RCC_SYSCLK_DIV16,     /*!< SYSCLK 16分频 */
    RCC_SYSCLK_DIV64,     /*!< SYSCLK 64分频 */
    RCC_SYSCLK_DIV128,    /*!< SYSCLK 128分频 */
    RCC_SYSCLK_DIV256,    /*!< SYSCLK 256分频 */
    RCC_SYSCLK_DIV512     /*!< SYSCLK 512分频 */
} rcc_sysclk_div_t;

/*!
 * @brief  系统时钟SYSCLK源
 */
typedef enum
{
    RCC_SYSCLK_SRC_HSI,
    RCC_SYSCLK_SRC_LSI = 3,
    RCC_SYSCLK_SRC_LSE
} rcc_sysclk_source_t;

/**
 * @brief MCO 时钟源
 */
typedef enum
{
    RCC_MCOSOURCE_NOCLOCK, /*!< MCO 输出关闭，无输出 */
    RCC_MCOSOURCE_SYSCLK,  /*!< SYSCLK 时钟输出 */
    RCC_MCOSOURCE_HSI = 3, /*!< HSI 时钟输出 */
    RCC_MCOSOURCE_LSI = 6, /*!< LSI 时钟输出 */
    RCC_MCOSOURCE_LSE      /*!< LSE 时钟输出 */
} rcc_mco_source_t;

/**
 * @brief MCO 时钟分频
 */
typedef enum
{
    RCC_MCODIV_1,  /*!< MCO 无分频 */
    RCC_MCODIV_2,  /*!< MCO 2分频 */
    RCC_MCODIV_4,  /*!< MCO 4分频 */
    RCC_MCODIV_8,  /*!< MCO 8分频 */
    RCC_MCODIV_16, /*!< MCO 16分频 */
    RCC_MCODIV_32, /*!< MCO 32分频 */
    RCC_MCODIV_64, /*!< MCO 64分频 */
    RCC_MCODIV_128 /*!< MCO 128分频 */
} rcc_mcodiv_t;

/*!
 * @brief  iwdg时钟选择
 */
typedef enum
{
    RCC_IWDG_SRC_LSI,
    RCC_IWDG_SRC_LSE
} rcc_iwdg_source_t;

/*!
 * @brief  awu时钟选择
 */
typedef enum
{
    RCC_AWU_SRC_LSI,
    RCC_AWU_SRC_LSE
} rcc_awu_source_t;

/*!
 * @brief  lcd时钟选择
 */
typedef enum
{
    RCC_LCD_SRC_LSI,
    RCC_LCD_SRC_LSE
} rcc_lcd_source_t;


/*!
 * @brief  LPUART时钟选择
 */
typedef enum
{
    RCC_LPUART_SRC_PCLK,
    RCC_LPUART_SRC_SYSCLK,
    RCC_LPUART_SRC_LSI,
    RCC_LPUART_SRC_LSE
} rcc_lpuart_source_t;

/*!
 * @brief  TIM时钟选择
 */
typedef enum
{
    RCC_TIM_SRC_HCLK,
    RCC_TIM_SRC_SYSCLK,
    RCC_TIM_SRC_48MHZ
} rcc_tim_source_t;

/*!
 * @brief  ADC时钟选择
 */
typedef enum
{
    RCC_ADC_SRC_SYSCLK,
    RCC_ADC_SRC_24MHZ
} rcc_adc_source_t;

/*!
 * @brief 模块时钟使能索引
 */
typedef enum
{
    RCC_CRC     = REG_IDX_BIT(RCC_REGIDX_AHB, RCC_AHBENR_CRCEN_Pos),
    RCC_GPIOA   = REG_IDX_BIT(RCC_REGIDX_AHB, RCC_AHBENR_GPIOAEN_Pos),
    RCC_GPIOB   = REG_IDX_BIT(RCC_REGIDX_AHB, RCC_AHBENR_GPIOBEN_Pos),
    RCC_GPIOC   = REG_IDX_BIT(RCC_REGIDX_AHB, RCC_AHBENR_GPIOCEN_Pos),

    RCC_TIM2    = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_TIM2EN_Pos),
    RCC_TIM6    = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_TIM6EN_Pos),
    RCC_TIM7    = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_TIM7EN_Pos),
    RCC_AWU     = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_AWUEN_Pos),
    RCC_WWDG    = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_WWDGEN_Pos),
    RCC_IWDG    = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_IWDGEN_Pos),
    RCC_LPUART1 = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_LPUART1EN_Pos),
    RCC_I2C1    = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_I2C1EN_Pos),
    RCC_PWR     = REG_IDX_BIT(RCC_REGIDX_APB1, RCC_APBENR1_PWREN_Pos),

    RCC_SYSCFG  = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_SYSCFGEN_Pos),
    RCC_SPI1    = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_SPI1EN_Pos),
    RCC_USART1  = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_USART1EN_Pos),
    RCC_TIM10   = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_TIM10EN_Pos),
    RCC_TIM16   = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_TIM16EN_Pos),
    RCC_ADC     = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_ADCEN_Pos),
    RCC_LCD     = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_LCDEN_Pos),
    RCC_TK      = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_TKEN_Pos),
    RCC_DBG     = REG_IDX_BIT(RCC_REGIDX_APB2, RCC_APBENR2_DBGEN_Pos)
} rcc_peripheral_t;
/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif
/*外设时钟使能*/
void RCC_EnablePeripheralClock(rcc_peripheral_t periph);
void RCC_DisablePeripheralClock(rcc_peripheral_t periph);
void RCC_PeripheralForceReset(rcc_peripheral_t periph);
void RCC_EnableOSC(rcc_osc_type_t osc);
void RCC_DisableOSC(rcc_osc_type_t osc);
void RCC_EnablePeripheralClockSleep(rcc_peripheral_t periph);
void RCC_DisablePeripheralClockSleep(rcc_peripheral_t periph);

/* 总线时钟配置和获取 */
uint32_t RCC_GetClocksFreq(rcc_clock_type_t clock);
rcc_sysclk_source_t RCC_GetSysClockSource(void);
void RCC_SetSysClockSource(rcc_sysclk_source_t clock);
void RCC_SetApb1Prescaler(rcc_hclk_div_t div);
void RCC_SetAhbPrescaler(rcc_sysclk_div_t div);
void RCC_SetHsiDiv(rcc_hsi_div_t div);
uint32_t RCC_GetApb1Prescaler(void);
uint32_t RCC_GetHsiDiv(void);
uint32_t RCC_GetAhbPrescaler(void);

/* MCO功能 */
void RCC_ConfigMco(rcc_mco_source_t mcoSrc, rcc_mcodiv_t mcoDiv);

/* 外设时钟源选择 */
void RCC_SetLpuart1ClockSource(rcc_lpuart_source_t src);
rcc_lpuart_source_t RCC_GetLpuart1ClockSource(void);
void RCC_SetAdcClockSource(rcc_adc_source_t src);
void RCC_SetLcdClockSource(rcc_lcd_source_t src);
void RCC_SetAwuClockSource(rcc_awu_source_t src);
void RCC_SetIwdgClockSource(rcc_iwdg_source_t src);

/* 复位原因 */
uint32_t RCC_GetResetReason(void);
void RCC_ClearAllResetFlag(void);

#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_RCC_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
