/*!
 * @file    nccm0670_spi.h
 * @brief   SPI固件库
 * @version 2021-04-28  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_SPI_H
#define NCC0670_SPI_H
#include "nccm0670.h"

/*!
 * @addtogroup nccm0670_spi
 * @{
 */
/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief SPI 分频值 范围为 2 ~ 256 之间的偶数分频
 */
#define SPI_BAUDRATE_PRESCALER(x)           (((x) - 2) / 2)

/*!
 * @brief SPI时钟相位配置
 */
typedef enum
{
    SPI_SCK_PHASE_1ST_EDGE, /*!< 在上升沿采样，下降沿发送 */
    SPI_SCK_PHASE_2ND_EDGE  /*!< 在下降沿采样, 上升沿发送 */
} spi_sck_phase_t;

/*!
 * @brief SPI时钟极性配置
 */
typedef enum
{
    SPI_SCK_ACTIVE_LOW,  /*!< SCK低电平有效 */
    SPI_SCK_ACTIVE_HIGH, /*!< SCK高电平有效 */
} spi_sck_polarity_t;

/*!
 * @brief SPI NSS片选极性配置
 */
typedef enum
{
    SPI_NSS_ACTIVE_LOW,
    SPI_NSS_ACTIVE_HIGH,
} spi_nss_polarity_t;
/*!
 * @brief SPI工作模式配置
 */
typedef enum
{
    SPI_PROTOCOL_MOTOROLA,  /*!< motorola模式 */
    SPI_PROTOCOL_TI = 2,        /*!< TI模式 */
    SPI_PROTOCOL_MICROWIRE, /*!< Microwire模式 */
} spi_protocol_t;

/*!
 * @brief SPI方向配置
 */
typedef enum
{
    SPI_DIRECTION_2LINES,        /*!< 双线全双工 */
    SPI_DIRECTION_1LINE,         /*!< 单线半双工 */
    SPI_DIRECTION_1LINE_RX_ONLY, /*!< 单线只接收 */
    SPI_DIRECTION_1LINE_TX_ONLY, /*!< 单线只发送 */
} spi_direction_t;

/*!
 * @brief 帧长度
 */
typedef enum
{
    SPI_FRAME_SIZE_8BITS  = 8u,  /*!< 帧长8位 */
    SPI_FRAME_SIZE_16BITS = 16u, /*!< 帧长16位 */
    SPI_FRAME_SIZE_32BITS = 0u,  /*!< 帧长32位 */
} spi_frame_size_t;

/*!
 * @brief 字节序配置
 */
typedef enum
{
    SPI_BYTE_ORDER_INTEL,    /*!< 低字节优先 */
    SPI_BYTE_ORDER_MOTOROLA, /*!< 高字节优先 */
} spi_byte_order_t;

/*!
 * @brief SPI主从模式
 */
typedef enum
{
    SPI_MODE_SLAVE,  /*!< 从机模式 */
    SPI_MODE_MASTER, /*!< 主机模式 */
} spi_mode_t;

/*!
 * @brief 软件硬件NSS
 */
typedef enum
{
    SPI_NSS_HARD, /*!< 硬件NSS */
    SPI_NSS_SOFT  /*!< 软件NSS */
} spi_nss_t;

/*!
 * @brief 位传输顺序
 */
typedef enum
{
    SPI_MSB_FIRST, /*!< 发送/接收数据时高位在前 */
    SPI_LSB_FIRST  /*!< 发送/接收数据时低位在前 */
} spi_bit_order_t;

/*!
 * @brief 滤波器
 */
typedef enum
{
    SPI_FILTER_ENABLE,   /*!< 使能滤波器 */
    SPI_FILTER_DISABLE /*!< 禁止滤波器 */
} spi_filter_t;

/*!
 * @brief SPI配置
 */
typedef struct
{
    spi_mode_t mode;                 /*!< 主从机模式 */
    spi_protocol_t protocol;       /*!< 帧模式 */
    spi_direction_t direction;       /*!< 传输方向 */
    spi_frame_size_t frame_size;     /*!< 帧长度 */
    spi_sck_polarity_t clk_polarity; /*!< SCK极性 */
    spi_sck_phase_t clk_phase;       /*!< SCK相位 */
    spi_nss_t nss;                   /*!< 软件NSS */
    spi_nss_polarity_t nss_polarity; /*!< 片选极性 */
    uint16_t prescaler;              /*!< 分频系数 */
    spi_bit_order_t bit_order;       /*!< LSB or MSB */
    spi_byte_order_t byte_order;     /*!< 字节序 */
    spi_filter_t filter;             /*!< 关闭采样滤波 */
} spi_config_t;

/*!
 * @brief 状态标志
 */
typedef enum
{
    SPI_FLAG_TXE  = SPI_SR_TXE_Msk,    /*!< 发送FIFO空标志 */
    SPI_FLAG_TXF  = SPI_SR_TXF_Msk,    /*!< 发送FIFO满标志 */
    SPI_FLAG_TXWR = SPI_SR_TXWR_Msk,   /*!< 发送FIFO低于水标志位*/
    SPI_FLAG_MMF  = SPI_SR_MMF_Msk,    /*!< 多主机错误标志 */
    SPI_FLAG_RXNE = SPI_SR_RXNE_Msk,   /*!< 接收FIFO非空标志 */
    SPI_FLAG_RXF  = SPI_SR_RXF_Msk,    /*!< 接收FIFO满标志 */
    SPI_FLAG_RXWR = SPI_SR_RXWR_Msk,   /*!< 接收FIFO超过水位标志 */
    SPI_FLAG_OVR  = SPI_SR_RXOVER_Msk, /*!< 接收溢出标志 */
    SPI_FLAG_FRE  = SPI_SR_FERR_Msk,   /*!< 帧错误标志 */
    SPI_FLAG_MAWU = SPI_SR_MAWU_Msk,   /*!< 匹配唤醒标志 */
    SPI_FLAG_BSY  = SPI_SR_BUSY_Msk,   /*!< 忙标志 */
    SPI_FLAG_TC   = SPI_SR_FEND_Msk,   /*!< 帧结束标志 */
} spi_status_flag_t;

/*!
 * @brief 中断使能
 */
typedef enum
{
    SPI_INT_TXE  = SPI_IER_TXEIE_Msk,  /*!< 发送FIFO空中断 */
    SPI_INT_TXF  = SPI_IER_TXFIE_Msk,  /*!< 发送FIFO满中断 */
    SPI_INT_TXWR = SPI_IER_TXWRIE_Msk, /*!< 发送FIFO低于水位中断 */
    SPI_INT_MMF  = SPI_IER_MMFIE_Msk,  /*!< 多主机错误中断 */
    SPI_INT_RXNE = SPI_IER_RXNEIE_Msk, /*!< 接收FIFO非空中断 */
    SPI_INT_RXF  = SPI_IER_RXFIE_Msk,  /*!< 接收FIFO满中断 */
    SPI_INT_RXWR = SPI_IER_RXWRIE_Msk, /*!< 接收FIFO超过水位中断 */
    SPI_INT_OVR  = SPI_IER_RXOVIE_Msk, /*!< 接收溢出中断 */
    SPI_INT_FRE  = SPI_IER_FERRIE_Msk, /*!< 帧错误中断 */
    SPI_INT_MAWU = SPI_IER_MAWUIE_Msk, /*!< 匹配唤醒中断 */
    SPI_INT_TC   = SPI_IER_FENDIE_Msk, /*!< 帧结束中断 */
} spi_interrupt_t;
/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

void SPI_DeInit(SPI_Type *const base);
void SPI_Init(SPI_Type *const base, const spi_config_t *const config);
void SPI_Enable(SPI_Type *const base);
void SPI_Disable(SPI_Type *const base);
void SPI_SetHalfDuplexTx(SPI_Type *const base, bool enable);
void SPI_SetMatchPattern(SPI_Type *const base, uint32_t pattern);
void SPI_SetMultipleMasterMode(SPI_Type *const base, bool enable);
void SPI_SetTxFIFOLevel(SPI_Type *const base, uint32_t level);
void SPI_SetRxFIFOLevel(SPI_Type *const base, uint32_t level);
void SPI_TransmitData(SPI_Type *const base, uint32_t data);
uint32_t SPI_ReceiveData(SPI_Type *const base);
void SPI_SetIntMode(SPI_Type *const base, spi_interrupt_t interrupt, bool enable);
bool SPI_GetIntFlag(SPI_Type *const base, spi_status_flag_t flag);
bool SPI_GetStatusFlag(SPI_Type *const base, spi_status_flag_t flag);
void SPI_ClearStatusFlag(SPI_Type *const base, spi_status_flag_t flag);

#if defined(__cplusplus)
}
#endif

/*! @} */
#endif /* NCC0670_SPI_H*/

/*******************************************************************************
 * EOF
 ******************************************************************************/
