/*! @file    nccm0670_syscfg.h
 *  @brief   SYSCFG固件库
 *  @version 2021-04-22  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_SYSCFG_H
#define NCC0670_SYSCFG_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_syscfg
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*!
 * @brief   定时器基本配置
 */
typedef enum
{
    SYSCFG_REMAP_FLASH,           /*!< 将主 Flash 映射到地址 0x0000 0000 */
    SYSCFG_REMAP_SRAM = 3         /*!< 将 SRAM 映射到地址 0x0000 0000 */
} syscfg_mem_remap_t;

/*!
 * @brief   IR极性选择
 */
typedef enum
{
    SYSCFG_IR_POL_INVERTED,    /*!< 反相*/
    SYSCFG_IR_POL_NONINVERTED, /*!< 非反相*/
} syscfg_ir_pol_t;

/*!
 * @brief   IR调制包络信号选择，TIM10 用于生成高频载波信号，而 TIM16（或者 USART1）生成调制包络
 */
typedef enum
{
    SYSCFG_IR_MODE_TIM16,   /*!< 采用TIM16生成调制包络 */
    SYSCFG_IR_MODE_USART1   /*!< 采用USART1生成调制包络 */
} syscfg_ir_mode_t;

/*!
 * @brief   TIM16触发源选择
 */
typedef enum
{
    SYSCFG_TIM16_TRG_SEL_NONE,
    SYSCFG_TIM16_TRG_SEL_TIM2 = 2,
    SYSCFG_TIM16_TRG_SEL_TIM6 ,
    SYSCFG_TIM16_TRG_SEL_TIM7,
    SYSCFG_TIM16_TRG_SEL_TIM10 = 6,
} syscfg_tim16_trg_selection_t;

/*!
 * @brief   TIM10触发源选择
 */
typedef enum
{
    SYSCFG_TIM10_TRG_SEL_NONE,
    SYSCFG_TIM10_TRG_SEL_TIM2 = 2,
    SYSCFG_TIM10_TRG_SEL_TIM6,
    SYSCFG_TIM10_TRG_SEL_TIM7,
    SYSCFG_TIM10_TRG_SEL_TIM16 = 6,
} syscfg_tim10_trg_selection_t;

/*!
 * @brief   TIM2触发源选择
 */
typedef enum
{
    SYSCFG_TIM2_TRG_SEL_NONE,
    SYSCFG_TIM2_TRG_SEL_TIM6 = 2,
    SYSCFG_TIM2_TRG_SEL_TIM7,
    SYSCFG_TIM2_TRG_SEL_TIM10 = 5,
    SYSCFG_TIM2_TRG_SEL_TIM16,
} syscfg_tim2_trg_selection_t;

/*!
 * @brief   ADC触发源选择
 */
typedef enum
{
    SYSCFG_ADC_TRG_SEL_TIM2 = 1,
    SYSCFG_ADC_TRG_SEL_TIM6,
    SYSCFG_ADC_TRG_SEL_TIM7,
    SYSCFG_ADC_TRG_SEL_TIM10 = 5,
    SYSCFG_ADC_TRG_SEL_TIM16,
    SYSCFG_ADC_TRG_SEL_EXTI11,
} syscfg_adc_trg_selection_t;

/*!
 * @brief NMI状态标志位
 */
typedef enum
{
    NMI_FLAG_SCM_ERR         = SYSCFG_NMISR_SCM_ERR_Msk,    /*!< 晶振监测异常 */
} syscfg_nmi_flag_t;

/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif
void SYSCFG_DeInit(SYSCFG_Type *const base);
void SYSCFG_SetRemapMemory(syscfg_mem_remap_t memory);
void SYSCFG_SetIrPolarity(syscfg_ir_pol_t polarity);
void SYSCFG_SetIrMode(syscfg_ir_mode_t mode);
void SYSCFG_SetAdcTriggerSelection(syscfg_adc_trg_selection_t source);
void SYSCFG_SetTim2TriggerSelection(syscfg_tim2_trg_selection_t source);
void SYSCFG_SetTim10TriggerSelection(syscfg_tim10_trg_selection_t source);
void SYSCFG_SetTim16TriggerSelection(syscfg_tim16_trg_selection_t source);
bool SYSCFG_GetNmiFlag(syscfg_nmi_flag_t flag);

#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_SYSCFG_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
