/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_btim BTIM
 * @brief BTIM模块固件库

## 详细说明 ##
- TIM6/TIM7为16位计数器 
- 重载寄存器位数和计数器相同
- 16位预分频器
- 计数器支持递增或递减模式
- 可配外部或系统时钟作为计数器计时时钟
- 可屏蔽的中断：更新中断，计数器上溢/下溢

## 使用指导 ##

@verbatim

#define BTIMx            TIM6
#define RCC_BTIMx        RCC_TIM6

    btim_config_t cfg = {
        .counter_mode = BTIM_COUNTER_DOWN,
        .period       = 9999,
        .prescaler    = 0,
    };
    RCC_EnablePeripheralClock(RCC_BTIMx);
    BTIM_DeInit(BTIMx);
    BTIM_Init(BTIMx, &cfg);
    BTIM_Enable(BTIMx);    

@endverbatim

 * @{
 */
#include "nccm0670_btim.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief  BTIM初始化
 *
 * 更新事件周期计算工式如下:
 * 更新周期 = TIM_CLK/((PSC + 1)*(ARR + 1))
 *     其中: TIM_CLK = timer clock input
 *           PSC = 16-bit prescaler register
 *           ARR = 16/32-bit Autoreload register
 *
 * @param[in] base  BTIM基址
 * @param[in] config 配置结构体指针
 * @return None
 ******************************************************************************/
void BTIM_Init(BTIM_Type *const base, const btim_config_t *const config)
{
    base->CR2 |= BTIM_CR2_CLKEN_Msk;
    /* 设置计数方向 */
    BTIM_SetCounterDirection(base, config->counter_mode);

    /* 设置自动重载值 */
    BTIM_SetAutoReload(base, config->period);

    /* 设置预分频器值 */
    BTIM_SetPrescaler(base, config->prescaler);

    /* 产生更新事件 */
    BTIM_GenerateEvent(base, BTIM_EVENT_SRC_UG);

    /*清更新标志*/
    BTIM_ClearStatusFlag(base, BTIM_FLAG_UIF);
}

/*!*****************************************************************************
 * @brief  BTIM反初始化
 *
 * @param[in] base  BTIM基址
 * @return None
 ******************************************************************************/
void BTIM_DeInit(BTIM_Type *const base)
{
    /* Force and Release reset on clock of base Port */
    if (base == TIM6)
    {
        RCC_PeripheralForceReset(RCC_TIM6);
    }
    else if (base == TIM7)
    {
        RCC_PeripheralForceReset(RCC_TIM7);
    }
    else
    {
    }
}

/*!*****************************************************************************
 * @brief  计数器计数方向
 *
 * @param[in] base  BTIM基址
 * @param[in] dir 递增or递减计数
 * @return None
 ******************************************************************************/
void BTIM_SetCounterDirection(BTIM_Type *const base, btim_counter_direction_t dir)
{
    base->CR1 &= ~(BTIM_CR1_DIR_Msk);
    base->CR1 |= BTIM_CR1_DIR(dir);
}

/*!*****************************************************************************
 * @brief  自动重载值配置
 *
 * @param[in] base  BTIM基址
 * @param[in] autoreload 自动重载值
 * @return None
 ******************************************************************************/
void BTIM_SetAutoReload(BTIM_Type *const base, uint32_t autoreload)
{
    base->ARR = autoreload;
}

/*!*****************************************************************************
 * @brief  自动重载预装载使能
 *
 * @param[in] base  BTIM基址
 * @param[in] enable 使能标志
 * @return None
 ******************************************************************************/
void BTIM_SetArrPreload(BTIM_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= BTIM_CR1_ARPE_Msk;
    }
    else
    {
        base->CR1 &= ~BTIM_CR1_ARPE_Msk;
    }
}

/*!*****************************************************************************
 * @brief  更新禁止
 *
 * @param[in] base  BTIM基址
 * @param[in] enable 使能标志
 * @return None
 ******************************************************************************/
void BTIM_SetUpdateEvent(BTIM_Type *const base, bool enable)
{
    if (!enable)
    {
        base->CR1 |= BTIM_CR1_UDIS_Msk;
    }
    else
    {
        base->CR1 &= ~BTIM_CR1_UDIS_Msk;
    }
}

/*!*****************************************************************************
 * @brief  获取计数器值
 *
 * @param[in] base  BTIM基址
 * @return 计数器值
 ******************************************************************************/
uint16_t BTIM_GetCounter(BTIM_Type *const base)
{
    return base->CNT;
}

/*!*****************************************************************************
 * @brief  预分频配置
 *
 * @param[in] base  BTIM基址
 * @param[in] prescaler 预分频器值
 * @return None
 ******************************************************************************/
void BTIM_SetPrescaler(BTIM_Type *const base, uint16_t prescaler)
{
    base->PSC = prescaler;
}

/*!*****************************************************************************
 * @brief  单脉冲模式配置
 *
 * @param[in] base  BTIM基址
 * @param[in] mode 单脉冲模式
 * @return None
 ******************************************************************************/
void BTIM_SetOnePulseMode(BTIM_Type *const base, btim_one_pulse_mode_t mode)
{
    base->CR1 &= ~BTIM_CR1_OPM_Msk;
    base->CR1 |= BTIM_CR1_OPM(mode);
}

/*!*****************************************************************************
 * @brief  更新请求源配置
 *
 * @param[in] base  BTIM基址
 * @param[in] source 更新请求源
 * @return None
 ******************************************************************************/
void BTIM_SetUpdateSource(BTIM_Type *const base, btim_update_request_src_t source)
{
    base->CR1 &= ~BTIM_CR1_URS_Msk;
    base->CR1 |= BTIM_CR1_URS(source);
}

/*!*****************************************************************************
 * @brief  BTIM模块使能
 *
 * @param[in] base  BTIM基址
 * @return None
 ******************************************************************************/
void BTIM_Enable(BTIM_Type *const base)
{
    base->CR1 |= BTIM_CR1_CEN_Msk;
}

/*!*****************************************************************************
 * @brief  BTIM模块失能
 *
 * @param[in] base  BTIM基址
 * @return None
 ******************************************************************************/
void BTIM_Disable(BTIM_Type *const base)
{
    base->CR1 &= ~BTIM_CR1_CEN_Msk;
}

/*!*****************************************************************************
 * @brief  更新请求源配置
 *
 * @param[in] base  BTIM基址
 * @param[in] event 事件生成源
 * @return None
 ******************************************************************************/
void BTIM_GenerateEvent(BTIM_Type *const base, btim_event_t event)
{
    base->EGR |= (uint32_t)event;
}

/*!*****************************************************************************
 * @brief  获取标志位
 *
 * @param[in] base  BTIM基址
 * @param[in] flag 标志
 * @return 标志位状态
 ******************************************************************************/
bool BTIM_GetStatusFlag(BTIM_Type *const base, btim_status_flag_t flag)
{
    return (base->SR1 & (uint32_t)flag) != 0U;
}

/*!*****************************************************************************
 * @brief  清除标志位
 *
 * @param[in] base  BTIM基址
 * @param[in] flag 标志
 * @return None
 ******************************************************************************/
void BTIM_ClearStatusFlag(BTIM_Type *const base, btim_status_flag_t flag)
{
    base->SR1 = (uint32_t)flag & 0xFFFFUL;
}

/*!*****************************************************************************
 * @brief  BTIM中断使能
 *
 * @param[in] base  BTIM基址
 * @param[in] interrupt 中断
 * @param[in] enable 使能
 * @return None
 ******************************************************************************/
void BTIM_SetIntMode(BTIM_Type *const base, btim_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        base->IER |= (uint32_t)interrupt;
    }
    else
    {
        base->IER &= ~(uint32_t)interrupt;
    }
}

/*!*****************************************************************************
 * @brief  获取中断标志位
 *
 * @param[in] base  BTIM基址
 * @param[in] flag 标志
 * @return 标志位状态
 ******************************************************************************/
bool BTIM_GetIntFlag(BTIM_Type *const base, btim_status_flag_t flag)
{
    return (base->SR1 & (uint32_t)flag) && (base->IER & (uint32_t)flag);
}

/**
 * @brief  设置主模式下将要发送到从定时器以实现同步的信息 (TRGO)
 *
 * @param[in] base  BTIM基址
 * @param[in] timer_synchronization 触发输出
 * @return None
 */
void BTIM_SetTriggerOutput(BTIM_Type *const base, btim_trigger_output_t timer_synchronization)
{
    base->CR2 &= ~BTIM_CR2_MMS_Msk;
    base->CR2 |= timer_synchronization;
}
/*! @} */
