/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_crc CRC
 * @brief CRC模块固件库

## 详细说明 ##
&emsp;&emsp;
CRC 计算单元具有单个 32 位读/写数据寄存器 (CRC_DR)。它用于输入新数据（写访问）和
保存之前 CRC 计算的结果（读访问）。对数据寄存器的每个写操作都会对之前的 CRC 值（存
储在 CRC_DR 中）和新值再做一次 CRC 计算。 CRC 计算针对整个 32 位数据字或逐个字节
完成，具体取决于数据的写入格式。 CRC_DR 寄存器可按字、右对齐半字和右对齐字节进行
访问。对于其它寄存器，只允许进行 32 位访问。

&emsp;&emsp;
输入数据的顺序可反转，以管理各种数据存放方式（双字/单字/字节，大端/小端等等）。可
对 8 位、 16 位和 32 位数据执行反转操作

CRC 支持以下几种：
- CRC-32-IEEE 802.3
- CRC-8: 8210
- CRC-16-CCIT
- CRC-16-IBM

## 使用指导 ##

@verbatim
    config.polynomial             = CRC_POLYNOMIAL_16_IBM;         // 多项式选择
    config.in_data_reverse        = CRC_IN_DATA_REVERSE_NONE;      // 输入数据翻转
    config.out_data_reverse       = CRC_OUT_DATA_REVERSE_NONE;     // 输出数据翻转
    config.default_init_value_use = CRC_DEFAULT_INIT_VALUE_ENABLE; // 使用默认的INIT值
    config.init_value             = 0x00000000;                    // 初始化值
    CRC_Init(&config);

    CRC_WriteData16(0x1234);    // 写入数据进行校验
    out_data = CRC_ReadData16();// 读出校验结果
@endverbatim


 * @{
 */
#include "nccm0670_crc.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief  CRC反初始化
 *
 * @return None
 ******************************************************************************/
void CRC_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_CRC);
}

/*!*****************************************************************************
 * @brief  CRC初始化
 *
 * @param[in] config 配置结构体
 * @return None
 ******************************************************************************/
void CRC_Init(const crc_config_t *config)
{
    /* CRC配置 */
    CRC->CR = CRC_CR_POLYSIZE(config->polynomial) | CRC_CR_REV_IN(config->in_data_reverse) |
              CRC_CR_REV_OUT(config->out_data_reverse);

    /* CRC初始值 */
    if (config->default_init_value_use == CRC_DEFAULT_INIT_VALUE_DISABLE)
    {
        CRC->INIT = config->init_value;
        CRC->CR |= CRC_CR_RESET(1);
    }
}

/*!*****************************************************************************
 * @brief  CRC写32位数据
 *
 * @param[in] data  要写入的数据
 * @return None
 ******************************************************************************/
void CRC_WriteData32(uint32_t data)
{
    CRC->DR = data;
}

/*!*****************************************************************************
 * @brief  CRC读32位数据
 *
 * @return  CRC校验结果
 ******************************************************************************/
uint32_t CRC_ReadData32(void)
{
    return (CRC->DR);
}

/*!*****************************************************************************
 * @brief  CRC写16位数据
 *
 * @param[in] data  要写入的数据
 * @return None
 ******************************************************************************/
void CRC_WriteData16(uint16_t data)
{
    *(uint16_t __IO *)(&CRC->DR) = data;
}

/*!*****************************************************************************
 * @brief  CRC读16位数据
 *
 * @return  CRC校验结果
 ******************************************************************************/
uint16_t CRC_ReadData16(void)
{
    return (uint16_t)(CRC->DR);
}

/*!*****************************************************************************
 * @brief  CRC写8位数据
 *
 * @param[in] data  要写入的数据
 * @return None
 ******************************************************************************/
void CRC_WriteData8(uint8_t data)
{
    *(uint8_t __IO *)(&CRC->DR) = data;
}

/*!*****************************************************************************
 * @brief  CRC读8位数据
 *
 * @return  CRC校验结果
 ******************************************************************************/
uint8_t CRC_ReadData8(void)
{
    return (uint8_t)(CRC->DR);
}
/*! @} */
