/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_dbg DBG
 * @brief DBG模块固件库

## 详细说明 ##

本模块用于仿真相关的配置：
- 休眠继续仿真
- 仿真暂停时冻结外设运行，如IWDG/AWU等

@verbatim

RCC_EnablePeripheralClock(RCC_DBG);
    DBG_FreezePeriph(DBG_BUZ_STOP);
    awu_gpio_init();
    RCC_EnableOSC(RCC_OSC_LSI);
    awu_config_t cfg ={
        .awu_timeout = 0,
        .buz_frequency = 455,
    };
    AWU_Init(&cfg);
    BUZ_Enable(); 
     
@endverbatim
 * @{
 */
#include "nccm0670_dbg.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief DBG反初始化
 ******************************************************************************/
void DBG_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_DBG);
}

/*!*****************************************************************************
 * @brief 在低功耗STOP模式下，DBG模块继续工作
 ******************************************************************************/
void DBG_EnableDbgStopMode(void)
{
    DBG->CR |= DBG_CR_DBG_STOP_Msk;
}

/*!
 * @brief 在低功耗STOP模式下，DBG模块不工作
 */
void DBG_DisableDbgStopMode(void)
{
    DBG->CR &= ~DBG_CR_DBG_STOP_Msk;
}

/*!*****************************************************************************
 * @brief 当内核停止时，冻结外设工作
 *
 ******************************************************************************/
void DBG_FreezePeriph(dbg_periph_t periph)
{
    REG32(DBG, REG_IDX(periph)) |= 1U << BIT_POS(periph);
}

/*!*****************************************************************************
 * @brief 当内核停止时，解除冻结外设工作
 *
 ******************************************************************************/
void DBG_UnFreezePeriph(dbg_periph_t periph)
{
    REG32(DBG, REG_IDX(periph)) &= ~(1U << BIT_POS(periph));
}
/*! @} */