/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_exti EXTI
 * @brief EXTI模块固件库

## 详细说明 ##

- 外部中断

## 使用指导 ##

@verbatim
    GPIO_SetPinMode(GPIOC, GPIO_MODE_INPUT, GPIO_PULLUP, GPIO_PIN_10);

    exti_cfg.line     = EXTI_LINE_10;
    exti_cfg.mode     = EXTI_MODE_INTERRUPT;
    exti_cfg.trigger  = EXTI_TRIGGER_RISING;
    exti_cfg.gpio_sel = EXTI_GPIOC;
    EXTI_SetConfigLine(&exti_cfg);
@endverbatim


 * @{
 */
#include "nccm0670_exti.h"

/*!*****************************************************************************
 * @brief EXTI 中断线配置
 *
 * @param[in] line 中断线号
 * @param[in] config EXTI中断线配置
 * @return None
 ******************************************************************************/
void EXTI_SetConfigLine(exti_line_t line, exti_config_t *config)
{
    uint32_t linepos;
    uint32_t maskline;

    linepos  = (uint32_t)line & EXTI_PIN_MASK;
    maskline = (1UL << linepos);

    /* 配置可配置中断线 */
    if (((uint32_t)line & EXTI_CONFIG) != 0x00U)
    {
        /* 配置上升沿触发 */
        if (config->trigger == EXTI_TRIGGER_RISING)
        {
            EXTI->RTSR |= maskline;
            EXTI->FTSR &= ~maskline;
        }
        /* 配置下降沿触发 */
        else if (config->trigger == EXTI_TRIGGER_FALLING)
        {
            EXTI->FTSR |= maskline;
            EXTI->RTSR &= ~maskline;
        }
        /* 配置上升下降沿触发 */
        else if (config->trigger == EXTI_TRIGGER_RISING_FALLING)
        {
            EXTI->RTSR |= maskline;
            EXTI->FTSR |= maskline;
        }
        else
        {
            EXTI->RTSR &= ~maskline;
            EXTI->FTSR &= ~maskline;
        }

        /* 如果为GPIO中断，配置GPIO端口选择 */
        if (((uint32_t)line & EXTI_GPIO) == EXTI_GPIO)
        {
            EXTI->EXTICR[linepos >> 2U] &= ~(EXTI_EXTICR_EXTI0_7_Msk << (8U * (linepos & 0x03U)));
            EXTI->EXTICR[linepos >> 2U] |= EXTI_EXTICR_EXTI0_7(config->gpio_sel) << (8U * (linepos & 0x03U));
        }
    }

    /* 配置中断模式 */
    if (config->mode == EXTI_MODE_INTERRUPT)
    {
        EXTI->IMR |= maskline;
        EXTI->EMR &= ~maskline;
    }
    /* 配置事件模式 */
    else if (config->mode == EXTI_MODE_EVENT)
    {
        EXTI->EMR |= maskline;
        EXTI->IMR &= ~maskline;
    }
    else
    {
        EXTI->IMR &= ~maskline;
        EXTI->EMR &= ~maskline;
    }
}

/*!*****************************************************************************
 * @brief 清除中断线配置
 *
 * @param[in] line 中断线号
 * @return None
 ******************************************************************************/
void EXTI_ClearConfigLine(exti_line_t line)
{
    uint32_t linepos;
    uint32_t maskline;

    linepos  = (uint32_t)line & EXTI_PIN_MASK;
    maskline = (1UL << linepos);

    /* 清除中断模式 */
    EXTI->IMR &= ~maskline;

    /* 清除事件模式 */
    EXTI->EMR &= ~maskline;

    /* 清除可配置中断的上升沿和下降沿 */
    if (((uint32_t)line & EXTI_CONFIG) != 0x00U)
    {
        EXTI->RTSR &= ~maskline;
        EXTI->FTSR &= ~maskline;

        /* 清除GPIO中断 */
        if (((uint32_t)line & EXTI_GPIO) == EXTI_GPIO)
        {
            EXTI->EXTICR[linepos >> 2U] &= ~(EXTI_EXTICR_EXTI0_7_Msk << (8U * (linepos & 0x03U)));
        }
    }
}

/*!*****************************************************************************
 * @brief 获取中断线是否挂起
 *
 * @param[in] line 中断线号
 * @return true：有中断，false：无中断
 ******************************************************************************/
bool EXTI_GetPending(exti_line_t line, exti_trigger_t edge)
{
    uint32_t regval;
    uint32_t linepos;

    linepos = (uint32_t)line & EXTI_PIN_MASK;

    if (edge != EXTI_TRIGGER_RISING)
    {
        /* 获取下降沿事件 */
        regval = EXTI->FPR;
    }
    else
    {
        /* 获取上升沿事件 */
        regval = EXTI->RPR;
    }

    return (regval & (1UL << linepos)) > 0U;
}

/*!*****************************************************************************
 * @brief 清除中断线挂起
 *
 * @param[in] line 中断线号
 * @param[in] edge 中断边沿
 *  @arg EXTI_TRIGGER_RISING
 *  @arg EXTI_TRIGGER_FALLING
 * @return None
 ******************************************************************************/
void EXTI_ClearPending(exti_line_t line, exti_trigger_t edge)
{
    uint32_t maskline;

    /* 计算中断掩码 */
    maskline = (1UL << ((uint32_t)line & EXTI_PIN_MASK));

    if (edge != EXTI_TRIGGER_RISING)
    {
        /* 清除下降沿事件 */
        EXTI->FPR = maskline;
    }
    else
    {
        /* 清除上升沿事件 */
        EXTI->RPR = maskline;
    }
}

/*!*****************************************************************************
 * @brief 生成软件中断
 *
 * @param[in] line 中断线号
 * @return None
 ******************************************************************************/
void EXTI_GenerateSwi(exti_line_t line)
{
    EXTI->SWIER = (1UL << ((uint32_t)line & EXTI_PIN_MASK));
}
/*! @} */
