/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_flash FLASH
 * @brief FLASH模块固件库

## 详细说明 ##
实现FLASH页擦除和字编程操作，支持IAP编程，不支持操作OPTION BYTE区

nccm0670页大小为4KB，操作时需要对齐

flash操作耗时较长，需要及时喂狗或者关闭看门狗

## 使用指导 ##

@verbatim
    FLASH_PageErase(0x8020000);
    FLASH_ProgramWord(0x8020000, 0x11223344);

@endverbatim


 * @{
 */
#include "nccm0670_flash.h"

#define FLASH_UNLOCK_KEY (0X653B0000U) /*!< 写保护解锁密钥 */
#define FLASH_TIMEOUT_COUNT (0xFFFFU)  /*!< FLASH操作超时计数  */

static bool FLASH_WaitForLastOperation(uint32_t timeout)
{
    uint32_t flash_status;
    bool retval = true;

    do
    {
        flash_status = FLASH->ISR;
        timeout--;
    } while ((flash_status & FLASH_ISR_BUSY_Msk) && !(flash_status & FLASH_ISR_OPERR_Msk) && (0U != timeout));

    if (timeout == 0 || flash_status > 0)
    {
        retval = false;
        /* 清除错误 */
        // FLASH->ISR |= FLASH_ISR_INTF_Msk;
    }

    return retval;
}

/*!*****************************************************************************
 * @brief 块擦除
 *
 * @param[in] page_address 块起始地址，4K对齐
 * @return 擦除结果
 * - true 成功
 * - false 失败
 ******************************************************************************/
bool FLASH_BlockErase(uint32_t page_adrress)
{
    /* 页起始地址 */
    FLASH->ADDR = page_adrress;
    FLASH->CTRL = FLASH_UNLOCK_KEY | FLASH_CTRL_OP_MODE(3) | FLASH_CTRL_START_Msk;

    __DSB();

    return FLASH_WaitForLastOperation(FLASH_TIMEOUT_COUNT);
}

/*!*****************************************************************************
 * @brief 页擦除
 *
 * @param[in] page_address 页起始地址，1K对齐
 * @return 擦除结果
 * - true 成功
 * - false 失败
 ******************************************************************************/
bool FLASH_PageErase(uint32_t page_adrress)
{
    /* 页起始地址 */
    FLASH->ADDR = page_adrress;
    FLASH->CTRL = FLASH_UNLOCK_KEY | FLASH_CTRL_OP_MODE(2) | FLASH_CTRL_START_Msk;

    __DSB();

    return FLASH_WaitForLastOperation(FLASH_TIMEOUT_COUNT);
}

/*!*****************************************************************************
 * @brief 字编程
 *
 * @param[in] address 编程地址，4字节对齐
 * @param[in] data 数据地址
 * @return 编程结果
 * - true 成功
 * - false 失败
 ******************************************************************************/
bool FLASH_ProgramWord(uint32_t address, uint32_t data)
{
    FLASH->ADDR = address;
    FLASH->CTRL = FLASH_UNLOCK_KEY | FLASH_CTRL_OP_MODE(1) | FLASH_CTRL_START_Msk;

    /* 写入数据 */
    FLASH->DATA = data;
    __DSB();

    return FLASH_WaitForLastOperation(FLASH_TIMEOUT_COUNT);
}

/*!*****************************************************************************
 * @brief 获取状态
 *
 * @param[in] flag 状态标志
 * @return 状态位
 * - true 置位
 * - false 复位
 ******************************************************************************/
bool FLASH_GetStatusFlag(flash_status_flag_t flag)
{
    return (FLASH->ISR >> FLASH_ISR_OPERR_Pos) == flag;
}

/*!*****************************************************************************
 * @brief 清除状态标志
 *
 * @param[in] flag 状态标志
 ******************************************************************************/
void FLASH_ClearStatusFlag(flash_status_flag_t flag)
{
    FLASH->ISR |= FLASH_ISR_INTF_Msk;
}
/*! @} */