/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_lcd LCD
 * @brief LCD模块固件库

## 详细说明 ##
- LCD驱动支持4*32（1/4占空比，1/3偏压）
- LED驱动支持4*32或8*28
- LCD驱动支持电阻型和电容型偏压产生电路
- 电阻型LCD驱动支持软件对比度调节和快速充电模式以降低功耗
- 电容型LCD驱动分为电容偏压型和内建电压稳定器型两种驱动方式
- 闪烁功能：可配置频率0.5/1/2/4Hz闪烁


## 使用指导 ##

@verbatim
    lcd_config_t config = {0};

    RCC_EnableOSC(RCC_OSC_LSI);
    RCC_EnablePeripheralClock(RCC_LCD);

    config.drive_mode = LCD_DRIVE_MODE_RESISTIVE_NETWORK_AUTO1;
    config.contrast   = LCD_CONTRAST_LEVEL_13;
    LCD_Init(&config);
    LCD_SetComAndSegMask(0xF, 0xfff, 0x0);
    LCD_Enable();
    LCD_Write(0, 0x0F);
@endverbatim


 * @{
 */
#include "nccm0670_lcd.h"
#include "nccm0670_rcc.h"

#define LCD_RAM_REGISTER_NUMBER (32)

/*!*****************************************************************************
 * @brief LCD反初始化
 *
 * @return None
 ******************************************************************************/
void LCD_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_LCD);
}

/*!*****************************************************************************
 * @brief LCD初始化
 *
 * @note 必须先打开 LSI 时钟后，才能使用LCD
 * @note 电容型LCD在初始化完成之后，需要等待至少25ms才能调用LCD_Enable()使能LCD
 * @note LCD驱动时只能使用1/4 duty， LED驱动时可以使用1/4或者1/8 duty
 * @param[in] config LCD配置
 * @return None
 ******************************************************************************/
void LCD_Init(const lcd_config_t *config)
{
    LCD->CR = config->drive_mode | LCD_CR_FCCTL(config->charge_time) | LCD_CR_VOL(config->contrast) |
              LCD_CR_DUTY(config->duty);

    LCD->FCR = LCD_FCR_BLINK(config->blink_mode) | LCD_FCR_BLINKF(config->blink_freq);

    LCD->CR |= LCD_CR_PUMPON(1);
}

/*!*****************************************************************************
 * @brief LCD模块使能
 *
 * @return None
 ******************************************************************************/
void LCD_Enable()
{
    LCD->CR |= LCD_CR_LCDON(1);
}

/*!*****************************************************************************
 * @brief LCD模块失能
 *
 * @return None
 ******************************************************************************/
void LCD_Disable()
{
    LCD->CR &= ~LCD_CR_LCDON_Msk;
}

/*!*****************************************************************************
 * @brief LCD闪烁配置
 *
 * @param[in] mode 闪烁模式
 * @param[in] freq 闪烁频率
 * @return None
 ******************************************************************************/
void LCD_ConfigBlink(lcd_blink_mode_t mode, lcd_blink_freq_t freq)
{
    LCD->FCR &= ~(LCD_FCR_BLINK_Msk | LCD_FCR_BLINKF_Msk);
    LCD->FCR |= LCD_FCR_BLINK(mode) | LCD_FCR_BLINKF(freq);
}

/*!*****************************************************************************
 * @brief LCD对比度配置
 *
 * @note 此接口不能用于电容型LCD驱动
 * @param[in] contrast 对比度
 * @return None
 ******************************************************************************/
void LCD_ConfigContrast(lcd_contrast_t contrast)
{
    LCD->CR &= ~LCD_CR_VOL_Msk;
    LCD->CR |= LCD_CR_VOL(contrast);
}

/*!*****************************************************************************
 * @brief 将数据写入LCD RAM中
 *
 * @param[in] regidx LCD RAM 索引
 * @param[in] data LCD 数据，取低5位或者低4位
 * @return None
 ******************************************************************************/
void LCD_Write(uint8_t regidx, uint8_t data)
{
    LCD->SDR[regidx] = (uint32_t)data;
}

/*!*****************************************************************************
 * @brief LCD清屏
 *
 * @return None
 ******************************************************************************/
void LCD_Clear(void)
{
    for (uint8_t index = 0; index < LCD_RAM_REGISTER_NUMBER; index++)
    {
        LCD->SDR[index] = 0;
    }
}

/*!*****************************************************************************
 * @brief LCD使能COM和SEG口
 *
 * @ note 设置LCD的COM和SEG掩码，只有相应位置1的时，对应的COM和SEG口才能使用
 * @param[in] com_mask COM口的掩码，bit[7-0]有效，分别对应COM8-COM1的使能
 * @param[in] seg_mask_low SEG口的掩码高位，bit[31-0]有效，分别对应SEG32-SEG1的使能
 * @param[in] seg_mask_high 预留
 * 
 * @note LCD驱动时只能使能4个COM，即com_mask=0xF; LED驱动时可以使能4个或者8个COM
 * @return None
 ******************************************************************************/
void LCD_SetComAndSegMask(uint32_t com_mask, uint32_t seg_mask_low, uint32_t seg_mask_high)
{
    (void)seg_mask_high;
    LCD->CENR  = com_mask;
    LCD->SENR1 = seg_mask_low;
}

/*!*****************************************************************************
 * @brief 设置LCD闪烁的像素点
 *
 * @param[in] com 要闪烁的com口，范围：0~7，对应COM1-COM8
 * @param[in] seg 要闪烁的seg口，范围：0~31，对应SEG1-SEG32
 * @return None
 ******************************************************************************/
void LCD_SetBlinkPixel(uint8_t com, uint8_t seg)
{
    LCD->FCR &= ~(LCD_FCR_BLINKCOM_Msk | LCD_FCR_BLINKSEG_Msk);
    LCD->FCR |= LCD_FCR_BLINKCOM(com) | LCD_FCR_BLINKSEG(seg);
}
/*! @} */
