/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_pwr PWR
 * @brief PWR模块固件库

## 详细说明 ##

- 两种低功耗模式：SLEEP和STOP
- STOP模式时可关闭FLASH、BANDGAP、LDO电源
- BOR(brown out reset)欠压复位功能
- 支持配置BOR阈值，及是否触发复位

## 使用指导 ##

@verbatim
    PWR_EnterStopMode(PWR_LOWPOWERREGULATOR_ON, PWR_LOWPOWER_ENTRY_WFI);

@endverbatim


 * @{
 */
#include "nccm0670_pwr.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief PWR反初始化
 *
 * @return None
 ******************************************************************************/
void PWR_DeInit(void)
{
    RCC_PeripheralForceReset(RCC_PWR);
}

/*!*****************************************************************************
 * @brief 进入休眠模式
 *
 * @param[in] sleep_entry 进入模式
 * @return None
 ******************************************************************************/
void PWR_EnterSleepMode(pwr_lowpower_entry_t sleep_entry)
{

    /* Clear SLEEPDEEP bit of Cortex System Control Register */
    SCB->SCR &= ~SCB_SCR_SLEEPDEEP_Msk;

    /* Select SLEEP mode entry -------------------------------------------------*/
    if (sleep_entry == PWR_LOWPOWER_ENTRY_WFI)
    {
        /* Request Wait For Interrupt */
        __WFI();
    }
    else
    {
        /* Request Wait For Event */
        __SEV();
        __WFE();
        __WFE();
    }
}

/*!*****************************************************************************
 * @brief 进入STOP模式
 *
 * @param[in] regulator LDO电源模式
 * @param[in] stop_entry 进入模式
 * @return None
 ******************************************************************************/
void PWR_EnterStopMode(pwr_regulator_t regulator, pwr_lowpower_entry_t stop_entry)
{
    PWR->CR &= ~(PWR_CR_HLDO_LPMOD_Msk | PWR_CR_LPDS_Msk);
    PWR->CR |= (uint32_t)regulator;

    SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;
    if (stop_entry == PWR_LOWPOWER_ENTRY_WFI)
    {
        __WFI();
    }
    else
    {
        __SEV();
        __WFE();
        __WFE();
    }
    SCB->SCR &= ~SCB_SCR_SLEEPDEEP_Msk;
}

/*!*****************************************************************************
 * @brief 低功耗模式下，ADC掉电
 * @param[in] enable 使能标志
 * @return None
 ******************************************************************************/
void PWR_SetAdcPowerDown(bool enable)
{
    if (enable)
    {
        PWR->CR |= PWR_CR_LP_MODE_Msk;
    }
    else
    {
        PWR->CR &= ~PWR_CR_LP_MODE_Msk;
    }
}

/*!*****************************************************************************
 * @brief STOP模式下，BANDGAP掉电
 * @param[in] enable 使能标志
 * @return None
 ******************************************************************************/
void PWR_SetBandgapPowerDown(bool enable)
{
    if (enable)
    {
        PWR->CR |= PWR_CR_BG_STOP_Msk;
    }
    else
    {
        PWR->CR &= ~PWR_CR_BG_STOP_Msk;
    }
}

/*!*****************************************************************************
 * @brief 设置欠压检测阈值
 *
 * @param[in] level 欠压检测阈值
 * @return None
 ******************************************************************************/
void PWR_SetBodLevel(pwr_bod_level_t bod_level)
{
    PWR->BORCR &= ~(PWR_BORCR_BOD_EN_Msk | PWR_BORCR_BOD_LEVEL_Msk);
    PWR->BORCR |= (uint32_t)bod_level;
}

/*!*****************************************************************************
 * @brief 设置欠压复位触发阈值
 *
 * @param[in] level 欠压复位阈值
 * @return None
 ******************************************************************************/
void PWR_SetBorLevel(pwr_bor_level_t bor_level)
{
    PWR->BORCR &= ~(PWR_BORCR_BOR_EN_Msk | PWR_BORCR_BOR_LEVEL_Msk);
    PWR->BORCR |= (uint32_t)bor_level;
}

/*! @} */
