/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
/*!
 * @defgroup nccm0670_rcc RCC
 * @brief RCC模块固件库

## 详细说明 ##

- 模块时钟开关配置
- 模块复位
- 总线时钟配置
- MCO功能配置
- 外设时钟选择

 * @{
 */
#include "nccm0670_rcc.h"

/*!
 * @name 外设时钟使能 API
 * @{
 */
/*!*****************************************************************************
 * @brief  打开外设时钟
 *
 * @param[in] periph 外设索引
 * @return None
 ******************************************************************************/
void RCC_EnablePeripheralClock(rcc_peripheral_t periph)
{
    REG32(RCC, (uint32_t)periph >> 24) |= 1U << BIT_POS(periph);
}

/*!*****************************************************************************
 * @brief  关闭外设时钟
 *
 * @param[in] periph 外设索引
 * @return None
 ******************************************************************************/
void RCC_DisablePeripheralClock(rcc_peripheral_t periph)
{
    REG32(RCC, ((uint32_t)periph >> 24) & 0xFF) &= ~(1U << BIT_POS(periph));
}

/*!*****************************************************************************
 * @brief  外设复位
 *
 * @param[in] periph 外设索引
 * @return None
 ******************************************************************************/
void RCC_PeripheralForceReset(rcc_peripheral_t periph)
{
    REG32(RCC, ((uint32_t)periph >> 16) & 0xFF) |= 1U << BIT_POS(periph);
    REG32(RCC, ((uint32_t)periph >> 16) & 0xFF) &= ~(1U << BIT_POS(periph));
}

/*!*****************************************************************************
 * @brief  休眠时打开外设时钟
 *
 * @param[in] periph 外设索引
 * @return None
 ******************************************************************************/
void RCC_EnablePeripheralClockSleep(rcc_peripheral_t periph)
{
    REG32(RCC, ((uint32_t)periph >> 8) & 0xFF) |= 1U << BIT_POS(periph);
}

/*!*****************************************************************************
 * @brief  休眠时关闭外设时钟
 *
 * @param[in] periph 外设索引
 * @return None
 ******************************************************************************/
void RCC_DisablePeripheralClockSleep(rcc_peripheral_t periph)
{
    REG32(RCC, ((uint32_t)periph >> 8) & 0xFF) &= ~(1U << BIT_POS(periph));
}

/*!*****************************************************************************
 * @brief  使能时钟源
 *
 * @param[in] osc 时钟源
 * @return None
 ******************************************************************************/
void RCC_EnableOSC(rcc_osc_type_t osc)
{
    REG32(RCC, REG_IDX(osc)) |= 1U << BIT_POS(osc);
}

/*!*****************************************************************************
 * @brief  失能时钟源
 *
 * @param[in] osc 时钟源
 * @return None
 ******************************************************************************/
void RCC_DisableOSC(rcc_osc_type_t osc)
{
    REG32(RCC, REG_IDX(osc)) &= ~(1U << BIT_POS(osc));
}
/*! @} */

/*!
 * @name 总线时钟配置和获取 API
 * @{
 */
/*!*****************************************************************************
 * @brief  获取总线时钟频率值
 *
 * @param clock 总线时钟选择
 * @return 时钟频率值
 ******************************************************************************/
uint32_t RCC_GetClocksFreq(rcc_clock_type_t clock)
{
    uint32_t freq;
    switch (clock)
    {
    case RCC_CLOCK_SYSCLK:
        if (RCC_GetSysClockSource() == RCC_SYSCLK_SRC_HSI)
        {
            freq = HSI_VALUE / HSIDivTable[RCC_GetHsiDiv()];
        }
        else if (RCC_GetSysClockSource() == RCC_SYSCLK_SRC_LSI)
        {
            freq = LSI_VALUE;
        }
        else if (RCC_GetSysClockSource() == RCC_SYSCLK_SRC_LSE)
        {
            freq = LSE_VALUE;
        }
        else
        {
            freq = 0U;
        }
        break;
    case RCC_CLOCK_HCLK:
        SystemCoreClockUpdate();
        freq = SystemCoreClock;
        break;
    case RCC_CLOCK_PCLK1:
        SystemCoreClockUpdate();
        freq = SystemCoreClock >> APBPrescTable[RCC_GetApb1Prescaler()];
        break;
    default:
        freq = 0U;
        break;
    }
    return freq;
}

/*!*****************************************************************************
 * @brief  获取系统时钟SYSCLK源
 *
 * @return 系统时钟源
 ******************************************************************************/
rcc_sysclk_source_t RCC_GetSysClockSource(void)
{
    return (rcc_sysclk_source_t)((RCC->CFGR & RCC_CFGR_SWS_Msk) >> RCC_CFGR_SWS_Pos);
}

/*!*****************************************************************************
 * @brief  配置系统时钟SYSCLK源
 *
 * @param[in] clock 时钟源
 * @return None
 ******************************************************************************/
void RCC_SetSysClockSource(rcc_sysclk_source_t clock)
{
    RCC->CFGR &= ~(RCC_CFGR_SW_Msk);
    RCC->CFGR |= ((uint32_t)clock << RCC_CFGR_SW_Pos);
}
/*!*****************************************************************************
 * @brief  配置APB1时钟PCLK1分频系数
 *
 * @param[in] div 分步系数
 * @return None
 ******************************************************************************/
void RCC_SetApb1Prescaler(rcc_hclk_div_t div)
{
    RCC->CFGR &= ~(RCC_CFGR_PPRE_Msk);
    RCC->CFGR |= (uint32_t)div << RCC_CFGR_PPRE_Pos;
}

/*!*****************************************************************************
 * @brief  配置AHB时钟HCLK分频系数
 *
 * @param[in] div 分频系数
 * @return None
 ******************************************************************************/
void RCC_SetAhbPrescaler(rcc_sysclk_div_t div)
{
    RCC->CFGR &= ~(RCC_CFGR_HPRE_Msk);
    RCC->CFGR |= (uint32_t)div << RCC_CFGR_HPRE_Pos;
}

/*!*****************************************************************************
 * @brief  配置HSI时钟分频系数
 *
 * @param[in] div 分频系数
 * @return None
 ******************************************************************************/
void RCC_SetHsiDiv(rcc_hsi_div_t div)
{
    RCC->CR &= ~RCC_CR_HSIDIV_Msk;
    RCC->CR |= (uint32_t)div << RCC_CR_HSIDIV_Pos;
}

/*!*****************************************************************************
 * @brief  获取APB1时钟分频系数
 *
 * @return APB1时钟分频系数值
 ******************************************************************************/
uint32_t RCC_GetApb1Prescaler(void)
{
    return (RCC->CFGR & RCC_CFGR_PPRE_Msk) >> RCC_CFGR_PPRE_Pos;
}

/*!*****************************************************************************
 * @brief  获取HSI时钟分频系数
 *
 * @return HSI时钟分频系数值
 ******************************************************************************/
uint32_t RCC_GetHsiDiv(void)
{
    return (RCC->CR & RCC_CR_HSIDIV_Msk) >> RCC_CR_HSIDIV_Pos;
}

/*!*****************************************************************************
 * @brief  获取AHB时钟分频系数
 *
 * @return AHB时钟分频系数值
 ******************************************************************************/
uint32_t RCC_GetAhbPrescaler(void)
{
    return (RCC->CFGR & RCC_CFGR_HPRE_Msk) >> RCC_CFGR_HPRE_Pos;
}
/*! @} */

/*!
 * @name 外设时钟源选择 API
 * @{
 */
/*!*****************************************************************************
 * @brief  设置LPUART1时钟源
 *
 * @param[in] src  时钟源
 * @return None
 ******************************************************************************/
void RCC_SetLpuart1ClockSource(rcc_lpuart_source_t src)
{
    RCC->KSEL &= ~RCC_KSEL_LPUART1SEL_Msk;
    RCC->KSEL |= RCC_KSEL_LPUART1SEL(src);
}

/*!*****************************************************************************
 * @brief  获取LPUART1时钟源
 *
 * @return 时钟源
 ******************************************************************************/
rcc_lpuart_source_t RCC_GetLpuart1ClockSource(void)
{
    return (rcc_lpuart_source_t)((RCC->KSEL & RCC_KSEL_LPUART1SEL_Msk) >> RCC_KSEL_LPUART1SEL_Pos);
}

/*!*****************************************************************************
 * @brief  设置ADC时钟源
 *
 * @param[in] src  时钟源
 * @return None
 ******************************************************************************/
void RCC_SetAdcClockSource(rcc_adc_source_t src)
{
    RCC->KSEL &= ~RCC_KSEL_ADCSEL_Msk;
    RCC->KSEL |= RCC_KSEL_ADCSEL(src);
}

/*!*****************************************************************************
 * @brief  设置LCD时钟源
 *
 * @param[in] src  时钟源
 * @return None
 ******************************************************************************/
void RCC_SetLcdClockSource(rcc_lcd_source_t src)
{
    RCC->KSEL &= ~RCC_KSEL_LCDSEL_Msk;
    RCC->KSEL |= RCC_KSEL_LCDSEL(src);
}

/*!*****************************************************************************
 * @brief  设置AWU时钟源
 *
 * @param[in] src  时钟源
 * @return None
 ******************************************************************************/
void RCC_SetAwuClockSource(rcc_awu_source_t src)
{
    RCC->KSEL &= ~RCC_KSEL_AWUSEL_Msk;
    RCC->KSEL |= RCC_KSEL_AWUSEL(src);
}

/*!*****************************************************************************
 * @brief  设置IWDG时钟源
 *
 * @param[in] src  时钟源
 * @return None
 ******************************************************************************/
void RCC_SetIwdgClockSource(rcc_iwdg_source_t src)
{
    RCC->KSEL &= ~RCC_KSEL_IWDGSEL_Msk;
    RCC->KSEL |= RCC_KSEL_IWDGSEL(src);
}

/*! @} */

/*!
 * @name MCO功能 API
 * @{
 */
/*!*****************************************************************************
 * @brief  MCO配置
 *
 * @param[in] mcoSrc  输出时钟源
 * @param[in] mcoDiv  输出时钟源分步系数
 * @return None
 ******************************************************************************/
void RCC_ConfigMco(rcc_mco_source_t mcoSrc, rcc_mcodiv_t mcoDiv)
{
    RCC->CFGR &= ~(RCC_CFGR_MCOSEL_Msk | RCC_CFGR_MCOPRE_Msk);
    RCC->CFGR |= ((uint32_t)mcoSrc << RCC_CFGR_MCOSEL_Pos) | ((uint32_t)mcoDiv << RCC_CFGR_MCOPRE_Pos);
}
/*! @} */

/*!
 * @name 复位原因 API
 * @{
 */
/*!*****************************************************************************
 * @brief  读取复位原因
 *
 * @return 复位原因寄存器值
 ******************************************************************************/
uint32_t RCC_GetResetReason(void)
{
    return RCC->RSTID;
}

/*!*****************************************************************************
 * @brief  清除所有复位标志
 *
 * @return None
 ******************************************************************************/
void RCC_ClearAllResetFlag(void)
{
    RCC->RSTID |= RCC_RSTID_RMVF_Msk;
}
/*! @} */
/*! @} */