/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_spi SPI
 * @brief SPI模块固件库

## 详细说明 ##
低功耗SPI模块支持主机和从机模式。该模块支持与外部器件进行半双工、全双工串行通信。
SPI在一个合适的异步时钟下可以当主机/从机，此时BUS时钟可关闭。从机在没有外部或
BUS时钟时，可以工作在只接收模式，待接收数据匹配后唤醒系统

- 支持Motorola SPI，TI，Microwire协议
- 主机、从机可配
- 主机波特率可配，支持2-256的偶数分频
- 支持全双工和半双工模式
- 支持只接收模式、只发送模式
- 传输字长8-32位可配
- 独立的发送FIFO和接收FIFO，大小为32*4bits，
    当传输字长等于8bits时，FIFO深度自动配置为16；
    当传输字长为9~16位时，FIFO深度自动配置为8；
    当传输字长为17~32位时，FIFO深度自动配置为4
- 传输帧byte order可配
- 传输帧half-word order可配
- Motorola SPI模式的时钟极性、相位可配
- 片选NSS信号的极性可配
- 支持外部时钟
- 支持待机模式时，数据匹配唤醒功能
- 支持FIFO的空满中断以及达到水位线中断
- 可触发中断的从模式下的帧错误标志
- 可触发中断的多主模式下的主模式故障标志

## 使用指导 ##

@verbatim
    RCC_EnablePeripheralClock(RCC_SPI1);
    RCC_EnablePeripheralClock(RCC_GPIOA);

    gpio_cfg.pin         = GPIO_PIN_4 | GPIO_PIN_5 | GPIO_PIN_6 | GPIO_PIN_7;
    gpio_cfg.mode        = GPIO_MODE_ALTERNATE;
    gpio_cfg.speed       = GPIO_SPEED_FREQ_HIGH;
    gpio_cfg.output_type = GPIO_OUTPUT_PUSHPULL;
    gpio_cfg.pull        = GPIO_PULLDOWN;
    gpio_cfg.alternate   = GPIO_AF0_LPSPI1;
    GPIO_Init(GPIOB, &gpio_cfg);

    spi_cfg.mode           = SPI_MODE_MASTER;
    spi_cfg.protocol       = SPI_PROTOCOL_MOTOROLA;
    spi_cfg.direction      = SPI_DIRECTION_2LINES;
    spi_cfg.frame_size     = SPI_FRAME_SIZE_8BITS;
    spi_cfg.clk_polarity   = SPI_SCK_ACTIVE_LOW;
    spi_cfg.clk_phase      = SPI_SCK_PHASE_2ND_EDGE;
    spi_cfg.nss_soft       = false;
    spi_cfg.nss_polarity   = SPI_NSS_ACTIVE_LOW;
    spi_cfg.prescaler      = 0x0;
    spi_cfg.byte_order     = SPI_BYTE_ORDER_MOTOROLA;
    spi_cfg.lsb_first      = false;
    spi_cfg.filter_disable = false;
    SPI_Init(SPI2, &spi_cfg);
@endverbatim
 * @{
 */
#include "nccm0670_spi.h"
#include "nccm0670_rcc.h"

/*!*****************************************************************************
 * @brief  SPI反初始化
 *
 * @param[in] base  SPI基址
 * @return None
 ******************************************************************************/
void SPI_DeInit(SPI_Type *const base)
{
    if (base == SPI1)
    {
        RCC_PeripheralForceReset(RCC_SPI1);
    }
}

/*!*****************************************************************************
 * @brief  SPI初始化
 *
 * @param[in] base  SPI基址
 * @param[in] config 配置结构体
 * @return None
 ******************************************************************************/
void SPI_Init(SPI_Type *const base, const spi_config_t *const config)
{
    base->CR2 = SPI_CR2_FCLKEN_Msk;
    /* 复位模块 */
    base->RSTR = SPI_RSTR_GRST_Msk;

    /* CR2在CR1前配置，防止在配置时输出波形 */
    base->CR2 |= SPI_CR2_CPOL(config->clk_polarity) | SPI_CR2_CPHA(config->clk_phase) | SPI_CR2_SSM(config->nss) |
                 SPI_CR2_SSI(config->nss_polarity) | SPI_CR2_LSBFIRST(config->bit_order) |
                 SPI_CR2_LENGTH(config->frame_size) | SPI_CR2_REORDER(config->byte_order);

    base->CR1 = SPI_CR1_FLTCL(config->filter) | SPI_CR1_TMOD(config->direction) | SPI_CR1_FRF(config->protocol) |
                 SPI_CR1_SSPOL(config->nss_polarity) | SPI_CR1_MSTR(config->mode);

    base->DIVR = SPI_DIVR_SCKDIV(config->prescaler);
}

/*!*****************************************************************************
 * @brief  使能SPI模块
 *
 * @param[in] base  SPI基址
 * @return None
 ******************************************************************************/
void SPI_Enable(SPI_Type *const base)
{
    base->ENR |= SPI_ENR_SPE_Msk;
}

/*!*****************************************************************************
 * @brief  失能SPI模块
 *
 * @param[in] base  SPI基址
 * @return None
 ******************************************************************************/
void SPI_Disable(SPI_Type *const base)
{
    base->ENR &= ~SPI_ENR_SPE_Msk;
}

/*!*****************************************************************************
 * @brief  半双工模式时传输方向设置
 *
 * @param[in] base  SPI基址
 * @param[in] enable ENABLE 发送，DISABLE 接收
 * @return None
 ******************************************************************************/
void SPI_SetHalfDuplexTx(SPI_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR1 |= SPI_CR1_HDDIR_Msk;
    }
    else
    {
        base->CR1 &= ~SPI_CR1_HDDIR_Msk;
    }
}

/*!*****************************************************************************
 * @brief  匹配唤醒设置
 *
 * @param[in] base  SPI基址
 * @param[in] pattern 唤醒匹配数据
 * @return None
 ******************************************************************************/
void SPI_SetMatchPattern(SPI_Type *const base, uint32_t pattern)
{
    base->DMR = pattern;   
}

/*!*****************************************************************************
 * @brief  发送多主机模式选择
 *
 * @param[in] base  SPI基址
 * @param[in] enable ENABLE 使能，DISABLE 禁止
 * @return None
 ******************************************************************************/
void SPI_SetMultipleMasterMode(SPI_Type *const base, bool enable)
{
    if (enable)
    {
        base->CR2 |= SPI_CR2_MMSEL_Msk;
    }
    else
    {
        base->CR2 &= ~SPI_CR2_MMSEL_Msk;
    }
}

/*!*****************************************************************************
 * @brief  发送FIFO水位设置
 *
 * @param[in] base  SPI基址
 * @param[in] level 水位
 * @return None
 ******************************************************************************/
void SPI_SetTxFIFOLevel(SPI_Type *const base, uint32_t level)
{
    base->FCR &= ~SPI_FCR_TX_WATER_Msk;
    base->FCR |= SPI_FCR_TX_WATER(level);
}

/*!*****************************************************************************
 * @brief  接收FIFO水位设置
 *
 * @param[in] base  SPI基址
 * @param[in] level 水位
 * @return None
 ******************************************************************************/
void SPI_SetRxFIFOLevel(SPI_Type *const base, uint32_t level)
{
    base->FCR &= ~SPI_FCR_RX_WATER_Msk;
    base->FCR |= SPI_FCR_RX_WATER(level);
}
/*!*****************************************************************************
 * @brief  发送数据
 *
 * @param[in] base  SPI基址
 * @param[in] data 待发送数据
 * @return None
 ******************************************************************************/
void SPI_TransmitData(SPI_Type *const base, uint32_t data)
{
    base->TDR = data;
}

/*!*****************************************************************************
 * @brief  获取接收到的数据
 *
 * @param[in] base  SPI基址
 * @return 接收到的数据
 ******************************************************************************/
uint32_t SPI_ReceiveData(SPI_Type *const base)
{
    return base->RDR;
}

/*!*****************************************************************************
 * @brief  开关中断
 *
 * @param[in] base  SPI基址
 * @param[in] interrupt 中断选择
 * @param[in] enable 开关标志 ENABLE/DISABLE
 * @return None
 ******************************************************************************/
void SPI_SetIntMode(SPI_Type *const base, spi_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        base->IER |= (uint32_t)interrupt;
    }
    else
    {
        base->IER &= ~(uint32_t)interrupt;
    }
}

/*!*****************************************************************************
 * @brief  获取中断状态
 *
 * @param[in] base  SPI基址
 * @param[in] flag 状态标志
 * - 不支持 SPI_FLAG_BSY
 * @return 中断状态位标志
 * - ENABLE   置位
 * - DISABLE  未置位
 ******************************************************************************/
bool SPI_GetIntFlag(SPI_Type *const base, spi_status_flag_t flag)
{
    return (base->IER & (base->SR) & (uint32_t)flag) > 0U;
}

/*!*****************************************************************************
 * @brief  获取状态标志
 *
 * @param[in] base  SPI基址
 * @param[in] flag 状态标志
 * @return 状态位标志
 * - ENABLE   置位
 * - DISABLE  未置位
 ******************************************************************************/
bool SPI_GetStatusFlag(SPI_Type *const base, spi_status_flag_t flag)
{
    return (base->SR & (uint32_t)flag) > 0U;
}

/*!*****************************************************************************
 * @brief  清除状态标志
 *
 * @param[in] base  SPI基址
 * @param[in] flag 状态标志
 * @return None
 ******************************************************************************/
void SPI_ClearStatusFlag(SPI_Type *const base, spi_status_flag_t flag)
{
    base->SR = (uint32_t)flag;
}
/*! @} */