/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
/*!
 * @defgroup nccm0670_usart USART
 * @brief USART模块固件库

## 详细说明 ##

- 全双工异步通信
- 采用NRZ 标准格式（标记/空格）
- 可配置为 16 倍过采样或 8 倍过采样
- 48 MHz 时钟频率和 8 倍过采样时，通用可编程收发波特率最高为 6 Mbps
- 自动波特率检测
- 数据字长度可编程（7 位、 8 位或 9 位）
- 可编程的数据顺序，最先移位 MSB 或 LSB
- 停止位可配置（支持 1 个或 2 个停止位）
- 单线半双工通信
- 支持8byte深度发送接收FIFO
- 发射器和接收器有单独的使能位
- 支持发送和接收的信号极性翻转
- 通信控制/错误检测标志
- 奇偶校验控制：
    - 发送奇偶校验位
    - 检查接收的数据字节的奇偶性
- 十六个具有标志位的中断源
- 支持多处理器通信，最高9位地址模式
- 从静默模式唤醒（通过空闲线检测或地址标记检测）
- LIN 主模式同步断路发送功能和 LIN 从模式断路检测功能
    - 当 USART 被配置为 LIN 使用时，可生成 13 位断路
- 支持 3/16 位持续时间的 IrDA SIR 编解码器
- 支持 ModBus 通信
    - 超时功能
    - CR/LF 字符识别


## 使用指导 ##

@verbatim
    usart_base_cfg_t USART_InitStruct;
    USART_InitStruct.baudrate              = 115200;                // 波特率
    USART_InitStruct.datawidth             = USART_DATAWIDTH_8B;    // 帧长度
    USART_InitStruct.stopbits              = USART_STOPBITS_1;      // 停止位
    USART_InitStruct.parity                = USART_PARITY_NONE;     // 奇偶校验位
    USART_InitStruct.transfer_direction    = USART_DIRECTION_TX_RX; // 传输方向
    USART_InitStruct.oversampling          = USART_OVERSAMPLING_16; // 过采样
    USART_Init(USART2, &USART_InitStruct);
    USART_Enable(USART2);
@endverbatim


 * @{
 */
#include "nccm0670_usart.h"
#include "nccm0670_rcc.h"

/*!
 * @name USART 通用 API
 * @{
 */
/*!
 * @brief USART反初始化
 *
 * @param[in] base 基地址
 * @return None
 */
void USART_DeInit(USART_Type *const base)
{
    if (base == USART1)
    {
        RCC_PeripheralForceReset(RCC_USART1);
    }
}

/*!
 * @brief USART初始化
 *
 * @param[in] base 基地址
 * @param[in] config USART配置
 * @return None
 */
void USART_Init(USART_Type *const base, const usart_base_cfg_t *config)
{
    base->CTL |= USART_CTL_UE_Msk;

    base->BR = (RCC_GetClocksFreq(RCC_CLOCK_PCLK1) / (config->baudrate));

    base->CTL &= ~(USART_CTL_ML_Msk | USART_CTL_STOP_Msk | USART_CTL_PCE_Msk | USART_CTL_PS_Msk | USART_CTL_TE_Msk |
                   USART_CTL_RE_Msk | USART_CTL_OVER8_Msk);
    base->CTL |= USART_CTL_ML(config->datawidth) | USART_CTL_STOP(config->stopbits) |
                 USART_CTL_OVER8(config->oversampling) | (uint32_t)(config->parity) |
                 (uint32_t)(config->transfer_direction);
}

/*!
 * @brief 设置波特率
 *
 * @param[in] base 基地址
 * @param[in] baud 波特率值
 * @return None
 */
void USART_SetBaudRate(USART_Type *const base, uint32_t baud)
{
    base->BR = (RCC_GetClocksFreq(RCC_CLOCK_PCLK1) / baud);
}

/*!
 * @brief USART使能
 */
void USART_Enable(USART_Type *const base)
{
    base->CTL |= USART_CTL_UE(1);
}

/*!
 * @brief USART禁止
 */
void USART_Disable(USART_Type *const base)
{
    base->CTL &= ~USART_CTL_UE_Msk;
}

/*!
 * @brief 设置传输方向
 */
void USART_SetTransferDirection(USART_Type *const base, usart_direction_t direction)
{
    base->CTL &= ~(USART_CTL_TE_Msk | USART_CTL_RE_Msk);
    base->CTL |= (uint32_t)(direction);
}

/*!
 * @brief 设置传输比特顺序(lsb或者msb)
 *
 * @param[in] base 基地址
 * @param[in] bitorder 比特顺序
 */
void USART_SetTransferBitOrder(USART_Type *const base, usart_bitorder_t bitorder)
{
    if (bitorder == USART_BITORDER_LSBFIRST)
    {
        base->CTL &= ~USART_CTL_MSBFIRST_Msk;
    }
    else
    {
        base->CTL |= USART_CTL_MSBFIRST(1);
    }
}

/*!
 * @brief 设置过采样
 *
 * @param[in] base 基地址
 * @param[in] oversampling 过采样
 */
void USART_SetOverSampling(USART_Type *const base, usart_oversampling_t oversampling)
{
    base->CTL &= ~USART_CTL_OVER8_Msk;
    base->CTL |= USART_CTL_OVER8(oversampling);
}

/*!
 * @brief 一个采样位方法使能
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetOneBitSample(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CTL |= USART_CTL_ONEBIT_Msk;
    }
    else
    {
        base->CTL &= ~USART_CTL_ONEBIT_Msk;
    }
}

/*!
 * @brief 设置二进制数据逻辑
 *
 * @param[in] base 基地址
 * @param[in] data_Logic 二进制数逻辑
 * @return None
 */
void USART_SetBinaryDataLogic(USART_Type *const base, usart_binary_logic_t data_Logic)
{
    base->CTL |= USART_CTL_DATAINV(data_Logic);
}

/*!
 * @brief 接收器超时使能
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetRxTimeoutMode(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CTL |= USART_CTL_RTOEN_Msk;
    }
    else
    {
        base->CTL &= ~USART_CTL_RTOEN_Msk;
    }
}

/*!
 * @brief 接收器超时值设置
 *
 * @param[in] base 基地址
 * @param[in] timeout 超时值
 * @return None
 */
void USART_SetRxTimeout(USART_Type *const base, uint32_t timeout)
{
    base->RTO &= ~USART_RTO_RTO_Msk;
    base->RTO |= USART_RTO_RTO(timeout);
}

/*!
 * @brief 发送数据值
 *
 * @param[in] base 基地址
 * @param[in] value 数据值
 * @return None
 */
void USART_TransmitData(USART_Type *const base, uint32_t value)
{
    base->TD = value & USART_TD_TDR_Msk;
}

/*!
 * @brief 接收数据值
 *
 * @param[in] base 基地址
 * @return None
 */
uint16_t USART_ReceiveData(USART_Type *const base)
{
    return (base->RD & USART_RD_RDR_Msk);
}

/*!
 * @brief 发送9位数据值
 *
 * @param[in] base 基地址
 * @param[in] value 数据值
 * @return None
 */
void USART_TransmitData9(USART_Type *const base, uint32_t value)
{
    base->CTL &= ~USART_CTL_TX_MBIT_Msk;
    base->CTL |= USART_CTL_TX_MBIT(value >> 8);

    base->TD = value & USART_TD_TDR_Msk;
}

/*!
 * @brief 接收9位数据值
 *
 * @param[in] base 基地址
 * @return None
 */
uint16_t USART_ReceiveData9(USART_Type *const base)
{
    uint16_t tmp;
    tmp = base->CTL & USART_CTL_RX_MBIT_Msk ? 0x100 : 0;
    return (base->RD & USART_RD_RDR_Msk) | tmp;
}

/*!
 * @brief 接收数据刷新请求
 *
 * @param[in] base 基地址
 * @return None
 */
void USART_RequestRxDataFlush(USART_Type *const base)
{
    base->RQ |= USART_RQ_RXFRQ(1);
}

/*!
 * @brief 发送数据刷新请求
 *
 * @param[in] base 基地址
 * @return None
 */
void USART_RequestTxDataFlush(USART_Type *const base)
{
    base->RQ |= USART_RQ_TXFRQ(1);
}
/*! @} */

/*!
 * @name 自动波特率侦测 API
 * @{
 */
/*!
 * @brief 自动波特率使能
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetAutoBaudRate(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CTL |= USART_CTL_ABREN_Msk;
    }
    else
    {
        base->CTL &= ~USART_CTL_ABREN_Msk;
    }
}

/*!
 * @brief 设置自动波特率模式
 *
 * @param[in] base 基地址
 * @param[in] mode 自动波特率模式
 * @return None
 */
void USART_SetAutoBaudRateMode(USART_Type *const base, usart_autobaud_mode_t mode)
{
    base->CTL &= ~USART_CTL_ABRMOD_Msk;
    base->CTL |= USART_CTL_ABRMOD(mode);
}

/*!
 * @brief 自动波特率请求
 *
 * @param[in] base 基地址
 * @return None
 */
void USART_RequestAutoBaudRate(USART_Type *const base)
{
    base->RQ |= USART_RQ_ABRRQ(1);
}
/*! @} */

/*!
 * @name 多机通信 API
 * @{
 */
/*!
 * @brief 设置USART节点的地址
 *
 * @note 在多处理器通信时于静默模式或低功耗模式下使用
 * @note 4位地址节点时，ADDM7=0,b7-b4位应设置为0
 *       8位地址节点时，ADDM7=1,发送器发送字符的 MSB 应为 1
 * @note 8位地址节点还可用于正常接收和静默模式无效时的字符检测（例如，ModBus协议中的块结束检测)。
 * 这种情况下，接收到的整个字符（8位）将与 ADD[7:0] 值进行比较，如果匹配， CMF 标志将置1
 *
 * @param[in] base 基地址
 * @param[in] address_len 地址长度
 * @param[in] node_address 节点地址
 * @return None
 */
void USART_ConfigNodeAddress(USART_Type *const base, usart_address_detect_t address_len, uint8_t node_address)
{
    base->CTL &= ~USART_CTL_ADDM7_Msk;
    base->CTL |= USART_CTL_ADDM7(address_len);

    base->CFG &= ~(USART_CFG_ADD7_Msk | USART_CFG_ADD3_Msk);
    base->CFG |= (((uint32_t)(node_address)) << USART_CFG_ADD7_Pos) & (USART_CFG_ADD7_Msk | USART_CFG_ADD3_Msk);
}

/*!
 * @brief 使能静默模式，USART 会按照 WAKE 位的定义在工作模式与静默模式之间切换
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetMute(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CTL |= USART_CTL_MME_Msk;
    }
    else
    {
        base->CTL &= ~USART_CTL_MME_Msk;
    }
}

/*!
 * @brief 静默模式请求
 *
 * @param[in] base 基地址
 * @return None
 */
void USART_RequestEnterMuteMode(USART_Type *const base)
{
    base->RQ |= USART_RQ_MMRQ(1);
}

/*!
 * @brief 设置USART静默模式的唤醒方法
 *
 * @param[in] base 基地址
 * @param[in] method 唤醒方法
 * @return None
 */
void USART_SetWakeUpMethod(USART_Type *const base, usart_wakeup_method_t method)
{
    base->CTL &= ~USART_CTL_WAKE_Msk;
    base->CTL |= USART_CTL_WAKE(method);
}
/*! @} */

/*!
 * @name LIN API
 * @{
 */
/*!
 * @brief 使能 LIN 模式
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetLin(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CFG |= USART_CFG_LINEN_Msk;
    }
    else
    {
        base->CFG &= ~USART_CFG_LINEN_Msk;
    }
}

/*!
 * @brief 立即请求在线路上发送 BREAK
 *
 * @param[in] base 基地址
 * @return None
 */
void USART_RequestBreakSending(USART_Type *const base)
{
    base->RQ |= USART_RQ_SBKRQ(1);
}

/*!
 * @name 半双工通信 API
 * @{
 */
/*!
 * @brief 使能单线半双工模式
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetHalfDuplex(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CTL |= USART_CTL_HDSEL_Msk;
    }
    else
    {
        base->CTL &= ~USART_CTL_HDSEL_Msk;
    }
}

/*!
 * @name 同步USART API
 * @{
 */
/*!
 * @brief 使能 SCLK 引脚时钟输出
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetSclkOutput(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CTL |= USART_CTL_CLKEN_Msk;
    }
    else
    {
        base->CTL &= ~USART_CTL_CLKEN_Msk;
    }
}

/*!
 * @brief USART同步时钟配置
 *
 * @param[in] base 基地址
 * @param[in] config 时钟配置
 * @return None
 */
void USART_ClockInit(USART_Type *const base, const usart_clock_cfg_t *config)
{
    base->CTL &= ~(USART_CTL_CLKEN_Msk | USART_CTL_CPOL_Msk | USART_CTL_CPHA_Msk | USART_CTL_LBCL_Msk);

    base->CTL |= USART_CTL_CLKEN(config->output) | USART_CTL_CPOL(config->polarity) |
                 USART_CTL_CPHA(config->phase) | USART_CTL_LBCL(config->last_clk_pulse);
}

/*!
 * @name Irda API
 * @{
 */

/*!
 * @brief 使能 IrDA 模式
 *
 * @param[in] base 基地址
 * @param[in] enable 使能
 * @return None
 */
void USART_SetIrda(USART_Type *const base, bool enable)
{
    if (enable)
    {
        base->CFG |= USART_CFG_IREN_Msk;
    }
    else
    {
        base->CFG &= ~USART_CFG_IREN_Msk;
    }
}

/*!
 * @brief 设置 IrDA 模式预分频值
 *
 * @param[in] base 基地址
 * @param[in] value 预分频值
 * @return None
 */
void USART_SetIrdaPrescaler(USART_Type *const base, uint8_t value)
{
    base->CFG &= ~USART_CFG_PSC_Msk;
    base->CFG |= USART_CFG_PSC(value);
}

/*!
 * @brief 设置 IrDA 低功耗模式
 *
 * @param[in] base 基地址
 * @param[in] mode 低功耗模式
 * @return None
 */
void USART_SetIrdaPowerMode(USART_Type *const base, usart_irda_power_mode_t mode)
{
    base->CFG &= ~USART_CFG_IRLP_Msk;
    base->CFG |= USART_CFG_IRLP(mode);
}
/*! @} */

/*!
 * @name 状态与中断 API
 * @{
 */
/*!
 * @brief 查询标志
 *
 * @note 若多个参数按位或，只要有一个标志被置位，函数就会返回true
 * @param[in] base 基地址
 * @param[in] flag 中断使能位，支持按位或
 * @return None
 */
bool USART_GetStatusFlag(USART_Type *const base, usart_status_flag_t flag)
{
    uint32_t msk = (uint32_t)flag;

    if (flag & USART_FLAG_RXNE)
    {
        msk |= USART_IS_RXNE_Msk;
    }
    else if (flag & USART_FLAG_TXE)
    {
        msk |= USART_IS_TXE_Msk;
    }
    else
    {
    }
    return base->IS & msk;
}

/*!
 * @brief 中断使能
 *
 * @param[in] base 基地址
 * @param[in] interrupt 中断
 * @param[in] enable 使能
 * @return None
 */
void USART_SetIntMode(USART_Type *const base, usart_interrupt_t interrupt, bool enable)
{
    if (enable)
    {
        base->IE |= (uint32_t)interrupt;
    }
    else
    {
        base->IE &= ~((uint32_t)interrupt);
    }
}

/*!
 * @brief 获取中断使能状态
 *
 * @param[in] base 基地址
 * @param[in] interrupt 要获取的中断位。
 * @return None
 */
bool USART_GetIntFlag(USART_Type *const base, usart_status_flag_t flag)
{
    return ((base->IE & (base->IS) & (uint32_t)flag) != 0);
}

/*!
 * @brief 清除标志位
 *
 * @param[in] base 基地址
 * @param[in] flag 要清除的状态位
 * @return None
 */
void USART_ClearStatusFlag(USART_Type *const base, usart_status_flag_t flag)
{
    base->IC = (uint32_t)flag;
}
/*! @} */
/*! @} */
