#include "i2c.h"
#include "nccm0670_i2c.h"
#include <stddef.h>

#define MAX_NBYTE_SIZE 255U

/**
 * @brief 等待状态位变化或者超时
 *
 * @param[in] i2cx             I2C基址
 * @param[in] flag             状态标志
 * @param[out] status          状态标志值
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, 1 - 超时
 */
static uint8_t i2c_wait_flag(I2C_Type *i2cx, i2c_status_flag_t flag, bool status, uint32_t timeout)
{
    while (I2C_GetStatusFlag(i2cx, flag) == status)
    {
         
        if (timeout == 0U)
        {
            I2C_AcknowledgeNextData(i2cx, I2C_NACK);
            return 1;
        }
        else
        {
            timeout--;
        }
    }
    return 0;
}

/**
 * @brief 读从设备命令,发送从设备地址和寄存器地址
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[in] mem_address      寄存器地址
 * @param[in] mem_address_size 寄存器地址长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
static uint8_t i2c_request_memory_read(I2C_Type *i2cx,
                                       uint16_t dev_address,
                                       uint16_t mem_address,
                                       uint16_t mem_address_size,
                                       uint32_t timeout)
{
    /* 发送从设备地址,请求写操作 */
    I2C_HandleTransfer(i2cx, dev_address, mem_address_size, I2C_MODE_SOFTEND, I2C_GENERATE_START_WRITE);
    /* 等待发送完成 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
    {
        return 1;
    }
    /* 如果寄存器地址为8位 */
    if (mem_address_size == 1U)
    {
        /* 发送寄存器地址 */
        I2C_TransmitData8(i2cx, mem_address);
    }
    /* 如果寄存器地址为16位 */
    else
    {
        /* 发送寄存器地址高8位 */
        I2C_TransmitData8(i2cx, mem_address >> 8U);
        /* 等待发送完成 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
        {
            return 2;
        }
        /* 发送寄存器地址低8位 */
        I2C_TransmitData8(i2cx, mem_address);
    }

    /* 等待TC置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_TC, false, timeout))
    {
        return 3;
    }

    return 0;
}

/**
 * @brief 写从设备命令,发送从设备地址和寄存器地址
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[in] mem_address      寄存器地址
 * @param[in] mem_address_size 寄存器地址长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
static uint8_t i2c_request_memory_write(I2C_Type *i2cx,
                                        uint16_t dev_address,
                                        uint16_t mem_address,
                                        uint16_t mem_address_size,
                                        uint32_t timeout)
{
    /* 发送从设备地址,请求写操作 */
    I2C_HandleTransfer(i2cx, dev_address, mem_address_size, I2C_MODE_RELOAD, I2C_GENERATE_START_WRITE);
    /* 等待发送完成 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
    {
        return 1;
    }
    /* 如果寄存器地址为8位 */
    if (mem_address_size == 1U)
    {
        /* 发送寄存器地址 */
        I2C_TransmitData8(i2cx, mem_address);
    }
    /* 如果寄存器地址为16位 */
    else
    {
        /* 发送寄存器地址高8位 */
        I2C_TransmitData8(i2cx, mem_address >> 8U);
        /* 等待发送完成 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
        {
            return 2;
        }
        /* 发送寄存器地址低8位 */
        I2C_TransmitData8(i2cx, mem_address);
    }
    /* 等待TCR置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
    {
        return 3;
    }

    return 0U;
}

/**
 * @brief 主设备发送数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_master_transmit(I2C_Type *i2cx, uint16_t dev_address, uint8_t *data, uint16_t size, uint32_t timeout)
{
    uint16_t xfer_count;
    uint16_t xfer_size;

    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 2;
    }

    xfer_count = size;

    /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (xfer_count > MAX_NBYTE_SIZE)
    {
        xfer_size = MAX_NBYTE_SIZE;
        /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE,并发送START和写请求 */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_START_WRITE);
    }
    else
    {
        xfer_size = xfer_count;
        /* 传输结果自动发送STOP, 并发送START和写请求 */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_START_WRITE);
    }

    while (xfer_count > 0U)
    {
        /* 等待发送完成 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout) != 0)
        {
            return 3;
        }
        /* 写入数据 */
        I2C_TransmitData8(i2cx, *data++);

        xfer_size--;
        xfer_count--;

        /* 当一组数据传输完, 需要传输下一组数据时 */
        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
            {
                return 4;
            }
            /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                /* 传输结果自动发送STOP */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_NOSTARTSTOP);
            }
        }
    }

    /* 等待STOP置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 5;
    }
    /* 清除STOP标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    return 0;
}

/**
 * @brief 主设备接收数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_master_receive(I2C_Type *i2cx, uint16_t dev_address, uint8_t *data, uint16_t size, uint32_t timeout)
{
    uint16_t xfer_count;
    uint16_t xfer_size;

    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 2;
    }

    xfer_count = size;

    /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (xfer_count > MAX_NBYTE_SIZE)
    {
        xfer_size = MAX_NBYTE_SIZE;
        /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE,并发送START和读请求 */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_START_READ);
    }
    else
    {
        xfer_size = xfer_count;
        /* 传输结果自动发送STOP, 并发送START和读请求 */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_START_READ);
    }

    while (xfer_count > 0U)
    {
        /* 等待RXNE置位 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_RXNE, false, timeout))
        {
            return 3;
        }
        /* 读取接收到的数据 */
        *data++ = I2C_ReceiveData8(i2cx);

        xfer_size--;
        xfer_count--;

        /* 当一组数据传输完, 需要传输下一组数据时 */
        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
            {
                return 4;
            }
            /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                /* 传输结果自动发送STOP */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_NOSTARTSTOP);
            }
        }
    }

    /* 等待STOP置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 5;
    }
    /* 清除STOP标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    return 0;
}

/**
 * @brief 从设备发送数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_slave_transmit(I2C_Type *i2cx, uint8_t *data, uint16_t size, uint32_t timeout)
{
    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 开启ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_ACK);
    /* 等待地址匹配 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_ADDR, false, timeout))
    {
        return 2;
    }
    /* 清除地址匹配标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_ADDR);
    /* 传输方向判断 */
    if (I2C_GetTransferDirection(i2cx) != I2C_DIRECTION_READ)
    {
        I2C_AcknowledgeNextData(i2cx, I2C_NACK);
        return 3;
    }

    while (size > 0U)
    {
        /* 等待允许写发送 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
        {
            return 4;
        }

        /* 写入数据 */
        I2C_TransmitData8(i2cx, *data);
        data++;
        size--;
    }

    /* 等待STOP置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 5;
    }
    /* 清除STOP标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);
    /* 等待BUSY清零 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 6;
    }
    /* 关闭ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_NACK);

    return 0;
}

/**
 * @brief 从设备接收数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_slave_receive(I2C_Type *i2cx, uint8_t *data, uint16_t size, uint32_t timeout)
{
    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 开启ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_ACK);
    /* 等待地址匹配 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_ADDR, false, timeout))
    {
        return 2;
    }
    /* 清除地址匹配标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_ADDR);
    /* 传输方向判断 */
    if (I2C_GetTransferDirection(i2cx) != I2C_DIRECTION_WRITE)
    {
        I2C_AcknowledgeNextData(i2cx, I2C_NACK);
        return 3;
    }

    while (size > 0U)
    {
        /* 等待接收到数据 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_RXNE, false, timeout))
        {
            return 4;
        }
        /* 保存接收到的数据 */
        *data = I2C_ReceiveData8(i2cx);
        data++;
        size--;
    }

    /* 等待STOP置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 5;
    }
    /* 清除STOP标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);
    /* 等待BUSY清零 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 6;
    }
    /* 关闭ACK */
    I2C_AcknowledgeNextData(i2cx, I2C_NACK);
    return 0;
}

/**
 * @brief 读取从设备数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[in] mem_address      寄存器地址
 * @param[in] mem_address_size 寄存器地址长度
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_mem_read(I2C_Type *i2cx,
                     uint16_t dev_address,
                     uint16_t mem_address,
                     uint16_t mem_address_size,
                     uint8_t *data,
                     uint16_t size,
                     uint32_t timeout)
{
    uint16_t xfer_count;
    uint16_t xfer_size;

    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 2;
    }

    /* 发送从设备地址和寄存器地址 */
    if (i2c_request_memory_read(i2cx, dev_address, mem_address, mem_address_size, timeout))
    {
        return 3;
    }

    xfer_count = size;
    /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (xfer_count > MAX_NBYTE_SIZE)
    {
        xfer_size = MAX_NBYTE_SIZE;
        /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE,并发送START和读请求 */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_START_READ);
    }
    else
    {
        xfer_size = xfer_count;
        /* 传输结果自动发送STOP, 并发送START和读请求 */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_START_READ);
    }

    do
    {
        /* 等待RXNE置位 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_RXNE, false, timeout))
        {
            return 4;
        }

        /* 读取接收到的数据 */
        *data++ = I2C_ReceiveData8(i2cx);

        xfer_size--;
        xfer_count--;

        /* 当一组数据传输完, 需要传输下一组数据时 */
        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
            {
                return 5;
            }
            /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                /* 传输结果自动发送STOP */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_NOSTARTSTOP);
            }
        }
    } while (xfer_count > 0U);

    /* 等待STOP置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 6;
    }
    /* 清除STOP标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    return 0;
}

/**
 * @brief 写入从设备数据
 *
 * @param[in] i2cx             I2C基址
 * @param[in] dev_address      从设备地址
 * @param[in] mem_address      寄存器地址
 * @param[in] mem_address_size 寄存器地址长度
 * @param[out] data            接收数据
 * @param[in] size             接收数据长度
 * @param[in] timeout          超时时间
 *
 * @retval 0 - 成功, other - 错误码
 */
uint8_t i2c_mem_write(I2C_Type *i2cx,
                      uint16_t dev_address,
                      uint16_t mem_address,
                      uint16_t mem_address_size,
                      uint8_t *data,
                      uint16_t size,
                      uint32_t timeout)
{
    uint16_t xfer_count;
    uint16_t xfer_size;

    /* 参数判断 */
    if (data == NULL || size == 0U)
    {
        return 1;
    }
    /* 等待总线空闲 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_BUSY, true, timeout))
    {
        return 2;
    }
    /* 发送从设备地址和寄存器地址 */
    if (i2c_request_memory_write(i2cx, dev_address, mem_address, mem_address_size, timeout))
    {
        return 3;
    }

    xfer_count = size;

    /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
    if (xfer_count > MAX_NBYTE_SIZE)
    {
        xfer_size = MAX_NBYTE_SIZE;
        /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_NOSTARTSTOP);
    }
    else
    {
        xfer_size = xfer_count;
        /* 传输结果自动发送STOP */
        I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_NOSTARTSTOP);
    }

    do
    {
        /* 等待TXIS标志置位 */
        if (i2c_wait_flag(i2cx, I2C_FLAG_TXIS, false, timeout))
        {
            return 4;
        }

        /* 写入数据 */
        I2C_TransmitData8(i2cx, *data++);

        xfer_count--;
        xfer_size--;

        /* 当一组数据传输完, 需要传输下一组数据时 */
        if ((xfer_count != 0U) && (xfer_size == 0U))
        {
            /* 等待传输完成 */
            if (i2c_wait_flag(i2cx, I2C_FLAG_TCR, false, timeout))
            {
                return 5;
            }
            /* 当数据长度大于 MAX_NBYTE_SIZE时需要分组传输 */
            if (xfer_count > MAX_NBYTE_SIZE)
            {
                xfer_size = MAX_NBYTE_SIZE;
                /* 当数据长度大于 MAX_NBYTE_SIZE时重载NBYTE */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_RELOAD, I2C_GENERATE_NOSTARTSTOP);
            }
            else
            {
                xfer_size = xfer_count;
                /* 传输结果自动发送STOP */
                I2C_HandleTransfer(i2cx, dev_address, xfer_size, I2C_MODE_AUTOEND, I2C_GENERATE_NOSTARTSTOP);
            }
        }

    } while (xfer_count > 0U);

    /* 等待STOP置位 */
    if (i2c_wait_flag(i2cx, I2C_FLAG_STOP, false, timeout))
    {
        return 6;
    }
    /* 清除STOP标志 */
    I2C_ClearStatusFlag(i2cx, I2C_FLAG_STOP);

    return 0;
}